
package carto.applet;

import carto.graphe.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import java.net.*;

/**
*  Cette classe correspond au panel sur lequel est dessin le graphe.
*  Elle gre galement le chargement du fichier XML  partir du serveur,
*  ainsi que la scurit des transferts.
*  @author Fred
*/

public class PanelGraphe extends JPanel implements MouseListener,Scrollable
{  
/**
*  Graphe qui est dessin sur notre panel.
	*/
	protected Graphe graphe;
	
	private Fenetre fenetre;
	private String nomFichierXML;
	private String precedentNomFichierXML;
	private int niveau; 
	private int difficulte;
	private int numChemin;
	private int index;
	private int style;
	
    /**
	*  Constructeur qui initialise un PanelGraphe
	*  @param fenetre Fentre dans laquelle le panel est destin  tre affich
	*  @param difficult Difficult dans laquelle on affiche le graphe
	*  @param numChemin Numero du chemin qui est affich avec le graphe
	*/
	
	public PanelGraphe(Fenetre fenetre,int difficulte,int numChemin)
	{
		super();
		addMouseListener(this);
		this.fenetre=fenetre;
		this.difficulte=difficulte;
		this.numChemin=numChemin;
	}
	
    /**
	*  Mthode qui initialise le graphe  partir du fichier XML, et du style de dessin prdfini.
	*/
	
	public void initGraphe()
	{
		graphe=CreeGraphe.initGraphe(nomFichierXML);
		graphe.setStyle(style);
	}
	
	
    // Ici gestion des action de l'utilisateur directement sur le graphe 
	
	public void mousePressed(MouseEvent event) 
	{}
	
	public void mouseReleased(MouseEvent event) 
	{}
	
	public void mouseEntered(MouseEvent event) 
	{}
	
	public void mouseExited(MouseEvent event) 
	{}
	
	public void mouseClicked(MouseEvent event) 
	{
		int X = event.getX();
		int Y = event.getY();
		Vector vecteur = graphe.getVecteurNoeud();
		for(int i = 0; i<vecteur.size(); i++)
		{
            Noeud noeud = (Noeud) vecteur.elementAt(i);
            if (noeud.dessin.getCouleur().equals(Color.red))
            {
				noeud.dessin.setCouleur(noeud.getDefautCouleur());
				fenetre.repaint(noeud.getCoordX()-noeud.dessin.getRayon(),noeud.getCoordY()-noeud.dessin.getRayon(),2*noeud.dessin.getRayon(),2*noeud.dessin.getRayon());
            }
            if (noeud.inside(X, Y))
            {
				noeud.dessin.setCouleur(Color.red);
				Central.actionUtilisateur(index,noeud.zoomer(),noeud.getURL(),event);
				fenetre.repaint(noeud.getCoordX()-noeud.dessin.getRayon(),noeud.getCoordY()-noeud.dessin.getRayon(),2*noeud.dessin.getRayon(),2*noeud.dessin.getRayon());
            }
		}
	}
	
    /**
	* Methode de l'interface Scrollable (donnes dans la doc online de java)
	*/
	
	public int getScrollableUnitIncrement(Rectangle visibleRect,
		int orientation,
		int direction) 
	{
		int maxUnitIncrement=10;
		//get the current position
		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL)
            currentPosition = visibleRect.x;
		else
            currentPosition = visibleRect.y;
		
		//return the number of pixels between currentPosition
		//and the nearest tick mark in the indicated direction
		if (direction < 0) 
		{
            int newPosition = currentPosition -
				(currentPosition / maxUnitIncrement) *
				maxUnitIncrement;
            return (newPosition == 0) ? maxUnitIncrement : newPosition;
		} 
		else 
		{
            return ((currentPosition / maxUnitIncrement) + 1) * 
				maxUnitIncrement - currentPosition;
		}
	}
	
    /**
	*  Mthode de l'interface Scrollable (doc online de Java)
	*/
	
	public int getScrollableBlockIncrement(Rectangle visibleRect,
		int orientation,
		int direction) 
	{
		int maxUnitIncrement=10;
		if (orientation == SwingConstants.HORIZONTAL)
            return visibleRect.width - maxUnitIncrement;
		else
            return visibleRect.height - maxUnitIncrement;
	}
	
    // Les methodes suivantes, toujours de l'interface Scrollable,
    // sont faites maison... pas trop dur quand meme !
	
	public boolean getScrollableTracksViewportHeight()
	{
		return false;
	}
	
	public boolean getScrollableTracksViewportWidth()
	{
		return false;
	}
	
	public Dimension getPreferredScrollableViewportSize()
	{
		return new Dimension(500,500);
	}
	
	
	// Acces aux attributs prive du panel
	
	public void setNomFichierXML(String nomFichierXML)
	{
		this.nomFichierXML=nomFichierXML;
	}
	
	public void setStyle(int style)
	{
		this.style=style;
	}
	
	public int getStyle( )
	{
		return style;
	}
	
	public void setNumChemin(int numChemin)
	{
		this.numChemin=numChemin;
	}
	
	public int getNumChemin()
	{
		return numChemin;
	}
	
	public void setDifficulte(int difficulte)
	{
		this.difficulte=difficulte;
	}
	
	public int getDifficulte()
	{
		return difficulte;
	}
	
	public int getIndex()
	{
		return index;
	}
	
	public void setIndex(int index)
	{
		this.index=index;
	}
	
    /**
	*  Retourne le nom du fichier XML du graphe de niveau suprieur  celui visualis actuellement dans le panel. Cette donne provient du fichier XML du graphe courrant.
	*/
	
	public String getPere()
	{
		return graphe.getPere();
	}
	
	public void paint(Graphics g)
	{
		//java.lang.NullPointerException, erreur au chargement du graphe (5/01/00)
		if(graphe!=null)
			graphe.dessine(g,difficulte,numChemin);
	}
}

