//Fleche.java

   package carto.dessin;
   import java.awt.*;
   import java.awt.geom.*;
/**Represente la difficulte d'un Lien Unidirectionnel */
   public class Fleche
   {
      static double angle = Math.PI/4;  
   
      protected Point pointdepart;
      protected Point extremehaut;
      protected Point extremebas;
   
      protected double norme;
      protected double vectX;
      protected double vectY;
   
   /**
   *Cree un Objet Fleche qui sera relie a une ligne
   *param x1,y1 coordonnees du point initial de la ligne
   *param x2,y2 coordonnees du point final de la ligne
   */
      public Fleche(double x1, double y1, double x2, double y2)
      {         
         vectX =-( x2 - x1);
         vectY =-( y2 - y1);
         norme = Math.sqrt(vectX * vectX + vectY * vectY);
      }
   
      public int valeur()
      {
         return 0;
      }
   
   /**
   *Calcule le point depart de une fleche individuel en fonction du decalage
   *@param x,y coordonnes du point principal depart d'une fleche
   *@param decalage indique le space entre chaque fleche individuel
   *@return le point depart de la fleche
   */
      public Point calculerPointDepart(double x, double y, int decalage)
      {
         if (norme != 0)
         {
            x = x + vectX * 5 * decalage / norme;
            y = y + vectY * 5 * decalage / norme;
         }
      
         return (new Point((int)x,(int)y));
      }
   
   /**
   *Calcule le point extreme haut d'une fleche individuel en fonction de 
   *l'angle que forme la ligne  de coordonnes depart x1,y1 et de 
   *coordonnees arrivee x2,y2 avec la horizontal
   *@param x1,y1 coordonnees depart de la ligne
   *@param x2,y2 coordonnees arrivee de la ligne
   *@return le point extreme haut de la fleche 
   */
      public Point calculerExtremeHaut(double x1, double y1, double x2, double y2)
      {
         int facteur = 15;
         double decalage_X, decalage_Y;
         double vect_x, vect_y;
         double normevecteur;  
      
      //on assigne les valeurs par defaut du depart du fleche   
         double extreme_x = x2;
         double extreme_y = y2;
      
      //on calcule les distances entre les deux points pour les axes X et Y.
         vect_x = x1 - x2;
         vect_y = y1 - y2;
      
         decalage_X = vect_x * Math.cos(angle) + vect_y * Math.sin(angle);
         decalage_Y = vect_y * Math.cos(angle) - vect_x * Math.sin(angle);
      
      // calcule la distance entre les deux noeuds pour
      // normaliser le vecteur calcule
         normevecteur = Math.sqrt(vect_x * vect_x + vect_y * vect_y);
      
         if (normevecteur != 0)
         {
            extreme_x = x2 + decalage_X * facteur / normevecteur;
            extreme_y = y2 + decalage_Y * facteur / normevecteur;
         }
         return (new Point((int)extreme_x,(int)extreme_y));
      }    
   
   
   /**
   *Calcule le point extreme bas d'une fleche individuel en fonction de 
   *l'angle que forme la ligne  de coordonnes depart x1,y1 et de 
   *coordonnees arrivee x2,y2 avec la horizontal
   *@param x1,y1 coordonnees depart de la ligne
   *@param x2,y2 coordonnees arrivee de la ligne
   *@return le point extreme bas de la fleche 
   */   
      public Point calculerExtremeBas(double x1, double y1, double x2, double y2)
      {
         int facteur = 15;
         double decalage_X, decalage_Y;
         double vect_x, vect_y;
         double normevecteur;  
      
      //on assigne les valeurs par defaut du depart du fleche   
         double extreme_x = x1;
         double extreme_y = y1;
      
      //on calcule les distances entre les deux points pour les axes X et Y.
         vect_x = x1 - x2;
         vect_y = y1 - y2;
      
         decalage_X = vect_x * Math.cos(angle) - vect_y * Math.sin(angle);
         decalage_Y = vect_y * Math.cos(angle) + vect_x * Math.sin(angle);
      
      // calcule la distance entre les deux noeuds pour
      // normaliser le vecteur calcule
         normevecteur = Math.sqrt(vect_x * vect_x + vect_y * vect_y);
      
         if (normevecteur != 0)
         {
            extreme_x = x2 + decalage_X * facteur / normevecteur;
            extreme_y = y2 + decalage_Y * facteur / normevecteur;
         
         }
         return new Point((int)extreme_x,(int)extreme_y);
      }    
   
   /**
   *Dessine une Fleche individuel 
   *@param g l'environement graphique
   *@param depart le point depart de la fleche individuel
   *@param haut le point extreme haut de la fleche
   *@param bas le point extreme bas de la fleche
   */
      public void dessinerFleche(Graphics g, Point depart, Point haut, Point bas)
      {  
         g.drawLine(depart.x, depart.y, haut.x, haut.y);
         g.drawLine(depart.x, depart.y, bas.x, bas.y);
      } 
   
   
      public void dessinerFleche(Graphics g)
      {
         System.out.println("Fleche");
      }
   
   }







