   package carto.dessin;

   import carto.graphe.*;
   import java.awt.*;
   import java.awt.geom.*;

/**
 * La reprsentation des lien.
 */
   public class Ligne extends Dessin
   {
      Lien lien;
      protected float epaisseur;
      
      /**
       * Les Points de Depart et Arrivee pour la ligne. 
       * Ce ne sont pas les coordonnes des noeuds.
       */
      protected int xD, yD, xA, yA; 
      
      protected float angle;
      protected Fleche fleche;
      protected Graphics2D g2;
      protected BasicStroke trait;
   
      FontMetrics fm;
   
   /**
   * Cre un objet ligne avec une trace, une paisseur et une couleur prcise.
   * @param lien le lien pour lequel la ligne est cree 
   */
   
      public Ligne(Lien lien)
      {this.lien=lien;}
      
      public Ligne(){
      	}// keiner weiss warum
   
      public void etablirDessin()
      {
         epaisseur = lien.getDefautEpaisseur();
         xD=lien.getNoeudD().getCoordX();
         yD=lien.getNoeudD().getCoordY();
         xA=lien.getNoeudA().getCoordX();
         yA=lien.getNoeudA().getCoordY();
         angle = setAngle();
         setPoints(); // des methodes differentes pour 'Ligne' et 'LigneChemin'
         trait = new BasicStroke(epaisseur,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND); 
         fleche=setFleche(((LienUniDir)lien).getDifficulte()); 
         //Claudia 21/07/99 - suppossons que le lien est toujours un 'LienUniDir'
      }
   
   /**
    * tablit un niveau de difficulte du Lien.
    * @param newDiff indique la nouvel difficulte du LienUniDir
    */
   
      protected Fleche setFleche(int newDiff) 
      {
         Fleche fleche=null;         
         switch(newDiff)
         {
            case 1: 
               fleche = new FlecheSimple(xD,yD,xA,yA);
               break;
            case 2: 
               fleche = new FlecheDouble(xD,yD,xA,yA);
               break;
            case 3: 
               fleche = new FlecheTriple(xD,yD,xA,yA);
         }
         return fleche;
      }
   
   /**
    * Modifie les coordonnees du centre du
    * noeud Depart pour tenir compte du rayon du noeud. 
    */ 
      protected void setPoints()
      {
      // pour le noeud de depart
         int rayonD = lien.getNoeudD().dessin.getRayon();
         if (xD > xA)
            xD -= rayonD * Math.cos(angle);
         else
            xD += rayonD * Math.cos(angle);
      
         if (yD > yA)
            yD -= rayonD * Math.sin(angle);
         else
            yD += rayonD * Math.sin(angle);
      
      
      // pour le noeud d'arrivee
         int rayonA = lien.getNoeudA().dessin.getRayon();
         if (xD > xA)
            xA += rayonA * Math.cos(angle);
         else
            xA -= rayonA * Math.cos(angle);
      
         if (yD > yA)
            yA += rayonA * Math.sin(angle);
         else
            yA -= rayonA * Math.sin(angle);
      }
   
    /**
     * Calcul de l'angle de la droite qui relie les 2 noeuds 
     * par rapport a l'horizontal. Valeurs entre 0 et pi/2
     */
   
      protected float setAngle()
      {
         float angle;
         if (xD != xA) 
         {
            angle = (float) Math.atan2(Math.abs(yA - yD),
                                       Math.abs(xA - xD));
            return angle;}
         else {
            angle = (float) Math.PI/2;
            return angle;
         }
      }
   
   /**
    * Dessine le symbole qui appartient a la ligne
    * @param g l'environement Graphique
    */
      protected void paintSymbole(Graphics g)
      {
         double xsymbole,ysymbole;
         double vectX, vectY, norme;
         int longsym = fm.stringWidth(lien.getType());
      
         vectX = -(xA - xD);
         vectY = -(yA - yD);
         norme = Math.sqrt(vectX * vectX + vectY * vectY);
         if (norme != 0)
         { 
            if (xD == xA)      // ligne |
            {
               ysymbole = xA + vectX * 30 / norme;
               xsymbole = yA + vectY * 30 / norme;
            }
            xsymbole = xA + vectX * 25 / norme;
            ysymbole = yA + vectY * 25 / norme;
         }
         else
         {
            xsymbole = xA;
            ysymbole = yA;
         }
         g.drawString(lien.getType(),(int) xsymbole,(int) ysymbole);
      } 
      
      protected FontMetrics etablirFont(Graphics g)
      {
         Font courierGras = new Font("Courier", Font.BOLD, 10);
         g.setFont(courierGras);
         return g.getFontMetrics(g.getFont());
      }
      
   /**
    * Dessine une ligne
    * @param g l'environement Graphique
    */
      public void paint(Graphics g)
      {
         g2 = (Graphics2D) g;
         g2.setColor(couleur);
         g2.drawLine(xD,yD,xA,yA);
         g2.setStroke(trait);
         Font fontCourante = g.getFont(); //pour memoriser?
         fm = etablirFont(g);
         paintSymbole(g);
         g.setFont(fontCourante);
         fleche.dessinerFleche(g);
      }
   }
