// *********************************************
// Modificications pour mise en libre :
// Modif C le 10/02/2000 par Marc Dauplait :
// Ajout de l'item de suppression de liens
// Modif de l'action Ouvrir et supprimerNoeud et de la mthode graphToModel
// pour ne pas pointer sur le mauvais graphe (clone)
// *********************************************

   package carto.editor;

   import java.util.Vector;

   import carto.graphe.*;
   import carto.dessin.*;
   import carto.parse.GrapheParser;
   import carto.parse.GraphWriter;

   import java.awt.*;
   import java.awt.event.*;
   import javax.swing.*;
   import javax.swing.event.*;
   import javax.swing.filechooser.FileFilter;
   import java.io.File;
   import java.io.FileOutputStream;
   import java.io.DataOutputStream;

   import diva.canvas.*;
   import diva.graph.*;
   import diva.graph.JGraph;
   import diva.graph.layout.*;
   import diva.graph.model.*;
   import diva.graph.bubble.*;

   import diva.canvas.connector.ConnectorTarget;
   import diva.canvas.connector.PerimeterSite;
   import diva.canvas.connector.Connector;
   import diva.canvas.connector.StraightConnector;
   import diva.canvas.connector.PerimeterTarget;
   import diva.canvas.connector.ArcConnector;
   import diva.canvas.event.LayerEvent;
   import diva.canvas.interactor.AbstractInteractor;
   import diva.canvas.toolbox.LabelWrapper;

/**
 * EditeurFrame is a JFrame that is used to display the graphs 
 * of the 'carto' package.
 * @author Claudia
 */
   public class EditeurFrame extends JFrame{
   
      JFrame frame;
      JGraph graph;
      GraphModel model;
      GraphView view;
      BasicGraphController bgc;
      GraphPane graphPane;
    /**
     * A pointer to the graph pane.
     */
      private LayoutTarget target = null;
   
    /**
     * Constructor -- create the Frame and give it a title.
     */
      public EditeurFrame(String title) {
         
         super(title);
         frame=this;
        // Create the menuBar and set it
         frame.setJMenuBar(createMenuBar());
      
         model = new GraphModel();
         bgc = new BasicGraphController(this);
         graph = new JGraph(model, bgc);
         graphPane = (GraphPane)graph.getCanvasPane();
         target = graphPane.getGraphView();
      	//BasicGraphController
         bgc.setConnectorTarget(new PerimeterTarget());
      
         view = graphPane.getGraphView();
         view.setIncrLayout(new RandomIncrLayout(view));
         view.setEdgeRenderer(new ArcRenderer());
         view.setNodeRenderer(new BubbleRenderer());
         getContentPane().add("Center", graph);
      
        // Close the window on any window event
         frame.addWindowListener(windowListener);
      }
   
   /** Create the menuBar
     */
      public JMenuBar createMenuBar () {
         JMenuBar menuBar;
         JMenu menuFile, menuNoeud, menuGraphe, menuLien, menuAide;
         JMenuItem itemNew, itemClose, itemSave, itemOuvrir, itemCreerG, 
         itemCreerL, itemModifieL, itemSupprimerL, itemCreerN, itemModifieN,
         itemSupprimerN, itemConculterAide;
      
       // Create the menuBar and menus
         menuBar = new JMenuBar();
      
         menuFile = new JMenu("Fichier");
         menuFile.setMnemonic('F');
      
         menuAide = new JMenu("Aide");
         menuAide.setMnemonic('A');
      
         menuGraphe = new JMenu("Graphe");
         menuGraphe.setMnemonic('G');
      
         menuNoeud = new JMenu("Noeud");
         menuNoeud.setMnemonic('N');
      
         menuLien = new JMenu("Lien");
         menuLien.setMnemonic('L');
      
        // Create the menu items
         itemNew = menuFile.add(actionNew);
         itemNew.setMnemonic('N');
         itemNew.setToolTipText("Efface l'diteur pour nouveau graphe");
      
         menuFile.addSeparator();
         itemOuvrir = menuFile.add(actionOuvrir);
         itemOuvrir.setMnemonic('O');
         itemOuvrir.setToolTipText("Ouvrir un fichier .xml");
      
         itemSave = menuFile.add(actionSave);
         itemSave.setMnemonic('S');
         itemSave.setToolTipText("Sauvgarder le graphe dans un fichier .xml");
      
         menuFile.addSeparator();
         itemClose = menuFile.add(actionClose);
         itemClose.setMnemonic('R');
         itemClose.setToolTipText("Fermer l'diteur");
      
         itemCreerG = menuGraphe.add(actionModifierGraphe);
         itemCreerG.setMnemonic('M');
         itemCreerG.setToolTipText("Modifier les donnes d'un Graphe");
      
         itemCreerN = menuNoeud.add(actionCreerNoeud);
         itemCreerN.setMnemonic('N');
         itemCreerN.setToolTipText("Crer nouveau Noeud");
      
         itemModifieN = menuNoeud.add(actionModifierNoeud);
         itemModifieN.setMnemonic('M');
         itemModifieN.setToolTipText("Modifie le noeud slectionn");
      
         itemSupprimerN = menuNoeud.add(actionSupprimerNoeud);
         itemSupprimerN.setMnemonic('S');
         itemSupprimerN.setToolTipText("Supprime le noeud slectionn");
      
         itemCreerL = menuLien.add(actionCreerLien);
         itemCreerL.setMnemonic('N');
         itemCreerL.setToolTipText("Crer nouveau Lien");

//         Pas implment pour l'instant
//         itemModifieL = menuLien.add(actionModifierLien);				// Modif C
//         itemModifieL.setMnemonic('M');									// Modif C
//         itemModifieL.setToolTipText("Modifie le Lien slectionn");	// Modif C
      
         itemSupprimerL = menuLien.add(actionSupprimerLien);			// Modif C
         itemSupprimerL.setMnemonic('S');								// Modif C
         itemSupprimerL.setToolTipText("Supprime le Lien slectionn");	// Modif C
       
         itemConculterAide = menuAide.add(actionAide);
         itemConculterAide.setMnemonic('C');
         itemConculterAide.setToolTipText("Consulter la documentation HTML");
      
      
        // Build the menus 
         menuBar.add(menuFile);
         menuBar.add(menuGraphe);
         menuBar.add(menuNoeud);
         menuBar.add(menuLien);
         menuBar.add(Box.createHorizontalGlue());
         menuBar.add(menuAide);
      
         return menuBar;
      }    
      
   
      public GraphModel getGraphModel()
      { 
         return model;
      }
      protected GraphView getGraphView()
      { 
         return view;
      }
      protected BasicGraphController getBasicGraphController()
      { 
         return bgc;
      }
   
    /////////////////////////////////////////////////////////////////
    // The action classes ///////////////////////////////////////////
    //
      private Action actionClose = 
         new AbstractAction ("Fermer") {
            public void actionPerformed(ActionEvent e) {
               System.exit(0);
            }
         };
   
      private Action actionModifierGraphe = 
      
         new AbstractAction ("Modifier ...") {
            public void actionPerformed(ActionEvent e) {
               FenetreMeta fmeta = new FenetreMeta((EditeurFrame)frame);
               }
         };
   
      private Action actionNew = 
         new AbstractAction ("Nouveau") {
            public void actionPerformed(ActionEvent e) {
               // si le GraphModel de 'graphPane' n'est pas vide => 
               // demande a enregistrer le graphe ?
               view.clear();
               GraphModel m = new GraphModel();
               graphPane.setGraphModel(m);
               model = m;
               FenetreMeta fmeta = new FenetreMeta((EditeurFrame)frame);
            }
         };
   
      private Action actionCreerNoeud = 
         new AbstractAction ("Nouveau ...") {
            public void actionPerformed(ActionEvent e) {
               FenetreNoeud fnoeud = new FenetreNoeud((EditeurFrame)frame);
            }
         };
   
      private Action actionModifierNoeud = 
         new AbstractAction ("Modifier ...") {
            public void actionPerformed(ActionEvent e) {
               Object obj = bgc.getSelectionModel().getLastSelection();
            	//LabelWrapper est la figure de nos noeuds
               if ( obj instanceof LabelWrapper){
                  LabelWrapper lw = (LabelWrapper)obj;
                  Node node = (Node) lw.getUserObject(); 
                  FenetreNoeud fnoeud = new FenetreNoeud((EditeurFrame)frame, (Noeud) node);
                 }
            }
         };
   
      private Action actionSupprimerNoeud = 
         new AbstractAction ("Supprimer ...") {
            public void actionPerformed(ActionEvent e) {
               Object obj = bgc.getSelectionModel().getLastSelection();
            	//LabelWrapper est la figure de nos noeuds
               if ( obj instanceof LabelWrapper){
                  LabelWrapper lw = (LabelWrapper)obj;
                  Node node = (Node) lw.getUserObject();
                  view.removeNodeMapping(node, lw);
                  lw.setVisible(false);
                  model.removeNode(node);
                  //mettre a jour le graphe - et le view
                  //model.setGraph((Graphe)node.getParent());					// Modif C
               }
            }
         };
   
      private Action actionCreerLien = 
      
         new AbstractAction ("Nouveau ...") {
            public void actionPerformed(ActionEvent e) {
            
               if ( ((Graphe)model.getGraph()).getVecteurNoeud().size() <= 1 ) {
                  JOptionPane jop = new JOptionPane();
                  jop.showMessageDialog(frame,"Le graphe ne contient pas de noeuds."
               	,"Warning",JOptionPane.ERROR_MESSAGE);
               }
               else{
                  FenetreLien flien = new FenetreLien((EditeurFrame)frame);
                  Edge lien = flien.getEdge();
                  if ( ((Lien)lien) instanceof LienUniDir ){
                     model.setEdgeHead(lien, ((Lien)lien).getNoeudA());
                     model.setEdgeTail(lien, ((Lien)lien).getNoeudD());
                     model.addEdge((LienUniDir)lien); 
                     //ca marche, parce que 'addEdge' dans 
                     //'carto.graphe.Graphe' appelle 'ajouterLienUni'
                  }
                  if ( ((Lien)lien) instanceof LienBiDir ){
                  // les deux liens sont ajoutes separement au modele
                     LienUniDir lien_1, lien_2;
                     lien_1=((LienBiDir)lien).premier();
                     model.setEdgeHead(lien_1, lien_1.getNoeudA());
                     model.setEdgeTail(lien_1, lien_1.getNoeudD());
                     model.addEdge(lien_1); 
                     lien_2=((LienBiDir)lien).deuxieme();
                     model.setEdgeHead(lien_2, lien_2.getNoeudA());
                     model.setEdgeTail(lien_2, lien_2.getNoeudD());
                     model.addEdge(lien_2);
                  }
               }
            }
         };

      //Pas implment pour l'instant

/*      private Action actionModifierLien = 									// Modif C
         new AbstractAction ("Modifier ...") {								// Modif C
            public void actionPerformed(ActionEvent e) {					// Modif C

            }
        };
*/

// Utile pour visualiser les liens en cours de debug. Je laisse pour celui
// qui ajoutera la fonction "Modifier Lien"
/*
 private void traceLiens() {
   Vector ve = ((Graphe)model.getGraph()).getVecteurLien();
   System.err.println("Taille vecteur = "+ve.size()+"\n");
   for (int i=0;i<ve.size();i++)
   {
   System.err.println((Lien)ve.elementAt(i)+"\n");
   }
   System.err.println("\n\n");
};
*/
  

      private Action actionSupprimerLien = 									// Modif C
         new AbstractAction ("Supprimer ...") {								// Modif C
            public void actionPerformed(ActionEvent e) {					// Modif C
               Object obj = bgc.getSelectionModel().getLastSelection();		// Modif C
            	//StraightConnector est la figure de nos liens				// Modif C
               if ( obj instanceof StraightConnector){						// Modif C
                  // recupere le Edge correspondant  la figure selectionnee// Modif C
                  StraightConnector sc = (StraightConnector)obj;			// Modif C
                  Edge edge = (Edge) sc.getUserObject();					// Modif C
                  // recupere le Lien correspondant au Edge					// Modif C
                  // manoeuvre complexe  cause du cas LienBiDir			// Modif C
                  Vector v = ((Graphe)model.getGraph()).getVecteurLien();	// Modif C
                  Noeud na = ((Lien)edge).getNoeudA();						// Modif C
                  Noeud nd = ((Lien)edge).getNoeudD();						// Modif C
                  Lien lien = (Lien)v.elementAt( ((Graphe)model.getGraph()).indexLien(nd,na) );// Modif C
                  // supression du Lien dans le vecteur de Graphe			// Modif C
                  ((Graphe)model.getGraph()).supprimerLien(lien);			// Modif C
                  // force la mise  jour de la vue selon le type de Lien	// Modif C
                  if ( lien instanceof LienUniDir ){						// Modif C
                     view.removeEdgeMapping(edge, sc);						// Modif C
                  }															// Modif C
                  if ( lien instanceof LienBiDir ){							// Modif C
                     // cas du LienBiDir : retrouver les 2 figures			// Modif C
                     // et forcer la mise  jour de la vue sur chaque		// Modif C
                     LienUniDir lien_1, lien_2;
                     lien_1=((LienBiDir)lien).premier();
                     view.removeEdgeMapping((Edge)lien_1, view.getEdgeFigure((Edge)lien_1));// Modif C
                     lien_2=((LienBiDir)lien).deuxieme();
                     view.removeEdgeMapping((Edge)lien_2, view.getEdgeFigure((Edge)lien_2));// Modif C
                  }
               }															// Modif C
            }																// Modif C
        };																	// Modif C

      private Action actionOuvrir = 
         new AbstractAction ("Ouvrir ...") {
            public void actionPerformed(ActionEvent e) {
               JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
               FileFilter xml = new XmlFileFiter();
               chooser.addChoosableFileFilter(xml);
               chooser.setFileFilter(xml);
            
               if(chooser.showOpenDialog((Component)(e.getSource()))
                  == JFileChooser.APPROVE_OPTION){
                  File file = chooser.getSelectedFile();
                  try {
                     debug("Open from: " + file.toURL() + "\n");
                  //carto.parse.GrapheParser()
                     GrapheParser parser = new GrapheParser();
                  
                  // recuperer la representation du graph
                     Graphe gr = parser.parse(file.toURL().toString());
                  
                     GraphModel m = graphToModel(gr);
                     graphPane.setGraphModel(m);
                     model = m;
                     //model.setGraph(gr);											// Modif C
                     frame.setTitle("Editeur de Graphe - " + gr.meta.getTitre());
                  }
                     catch(Exception ex) {
                        debug("ERROR: " + ex.getMessage());
                     }
               }
            }
         /** 
          * Ajoute tous les elements du graphe dans le modele. 
          */
            protected GraphModel graphToModel(Graphe graphe)
            {
               GraphModel model = new GraphModel();
               Node node;
               Edge edge;
               //LienChemin lc;
               //Chemin ch;
               //Vector vectLienCh;
            
               java.util.Vector vecteurNoeud, vecteurLien;
               vecteurNoeud = graphe.getVecteurNoeud();
               if (vecteurNoeud.size() > 0){
                  for (int i=0;i<vecteurNoeud.size();i++){
                     node=(Node) vecteurNoeud.elementAt(i);
                     //"label" sera afficher avec le noeud au diteur
                     node.setProperty("label",((Noeud)node).getIdent());
                     model.addNode(node);
                  }
               }
               vecteurLien = graphe.getVecteurLien();
               if (vecteurLien.size() > 0){
                  for (int i=0;i<vecteurLien.size();i++){
                     edge = (Lien) vecteurLien.elementAt(i);
                  
                     if ( edge instanceof LienUniDir ){
                        model.setEdgeHead(edge, ((Lien)edge).getNoeudA());
                        model.setEdgeTail(edge, ((Lien)edge).getNoeudD());
                        //"label" sera afficher avec le lien au diteur
                        edge.setProperty("label", ((Lien)edge).getType());
                        model.addEdge((LienUniDir)edge); 
                     //ca marche, parce que 'addEdge' dans 'carto.graphe.Graphe' appelle 'ajouterLienUni'
                     }
                     if ( edge instanceof LienBiDir ){
                     // les deux liens sont ajoutes separement au modele
                        LienUniDir lien_1, lien_2;
                        lien_1=((LienBiDir)edge).premier();
                        model.setEdgeHead(lien_1, lien_1.getNoeudA());
                        model.setEdgeTail(lien_1, lien_1.getNoeudD());
                        //"label" sera afficher avec le lien au diteur
                        lien_1.setProperty("label", ((Lien)lien_1).getType());
                        model.addEdge(lien_1); 
                        lien_2=((LienBiDir)edge).deuxieme();
                        model.setEdgeHead(lien_2, lien_2.getNoeudA());
                        model.setEdgeTail(lien_2, lien_2.getNoeudD());
                        //"label" sera afficher avec le lien au diteur
                        lien_2.setProperty("label", ((Lien)lien_2).getType());
                        model.addEdge(lien_2);
                     }
                  }
               
               }
               /* En attendant que l'dition des chemins soit implmente				// Modif C
               *  on transfert le vecteurChemin tel quel								// Modif C
               *  Pour cel on utilise la mthode setVecteurChemin cre spcialement	// Modif C
               *  pour ce cas temporaire. Il faudra donc la dtruire plus tard.			// Modif C
               */
               ((Graphe)model.getGraph()).setVecteurChemin(graphe.getVecteurChemin());	// Modif C
               /*if (!vecteurCheminVide){
                  for (int i=0;i<vecteurChemin.size();i++){
                     ch = (Chemin) vecteurChemin.elementAt(i);
                     vectLienCh = (Vector) ch.vectLienChemin;
                     if (!vectLienCh.equals(null)){
                        for (int j=0;j<vectLienCh.size();j++){
                           lc=(LienChemin) vectLienCh.elementAt(j);
                           lc.?;
                        } 
                     }
                  }  
               }*/
               return model;
            }
         };
   
      private Action actionSave = 
         new AbstractAction ("Sauvegarder sous ...") {
            public void actionPerformed(ActionEvent e) {
            
               JFileChooser chooser = new JFileChooser(System.getProperty("user.dir"));
               FileFilter xml = new XmlFileFiter();
               chooser.addChoosableFileFilter(xml);
               chooser.setFileFilter(xml);
            
               if(chooser.showSaveDialog((Component)(e.getSource()))
                  == JFileChooser.APPROVE_OPTION){
                  File file = chooser.getSelectedFile();
                  System.out.println(file.toString());
                  debug("Save to: " + file.getPath() + "\n");
                  try {
                     DataOutputStream out =
                     new DataOutputStream(new FileOutputStream(file.getPath()));
                     GraphWriter writer = new GraphWriter();
                     writer.write(model, out);
                  }
                     catch(Exception ex) {
                        debug("ERROR: " + ex.getMessage());
                     }
               }
            }
         };
   
      private Action actionAide = 
         new AbstractAction ("Consulter") {
            public void actionPerformed(ActionEvent e) {
            }
         };
   
    /**
     * Debugging output to standard err.
     */
      private void debug(String s) {
         System.err.println(s);
      }
   
      private class XmlFileFiter extends FileFilter {
      
         public boolean accept(File f) {
            if(f.isDirectory()) {
               return true;
            }
            String s = f.getName();
            int i = s.lastIndexOf('.');
            if(i > 0 &&  i < s.length() - 1) {
               String extension = s.substring(i+1).toLowerCase();
               return "xml".equals(extension);
            }
            return false;                   
         }
      
        // The desctiption of this filter
      
         public String getDescription() {
            return "Graph files (*.xml)";
         }
      }
   
    /////////////////////////////////////////////////////////////////
    // The window listener
    //
      WindowListener windowListener = 
         new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
               actionClose.actionPerformed(null);
            }
            public void windowIconified (WindowEvent e) {
               System.out.println(e);
            }
            public void windowDeiconified (WindowEvent e) {
               System.out.println(e);
            }
         };
   }
