   package carto.graphe;

   import java.io.*;
   import java.util.*;
   import java.awt.Graphics;

/**
 *  Cette classe dfinit un chemin d'un graphe.
 */

   public class Chemin
   {
    /**
     *  Vecteur qui contient tout les liens du chemin. Les liens sont de la classe LienChemin
     */
      protected Vector vectLienChemin;
      private Graphe graphePere;
      private int cours;
      private String nomChemin;
   
    /**
     * Constructeur qui initialise un chemin, avec un nom et un numro de cours donn. 
     * Son vecteur de liens est vide. Il faut en ajouter par la suite.
     */
      public Chemin(int cours,String nomChemin, Graphe graphe)
      {
         vectLienChemin=new Vector();
         this.cours=cours;
         this.nomChemin = nomChemin;
         graphePere = graphe;
      }
   
    /**
     *  Renvoie le numro de cours qui correspond au chemin.
     */   
      public int getCours()
      {
         return cours;
      }
   
    /**
     *  Renvoie le nom du chemin
     */
      public String getNomChemin()
      {
         return nomChemin;
      }
   
   /**
    * Donne une reprsentation String du chemin.
    * @author Claudia
    */
      public String toString()
      {
         String resu="Chemin : Cours "+cours+" - "+nomChemin+"\n";
         LienChemin l;
         for (int i=0;i<vectLienChemin.size();i++){
            l=(LienChemin) vectLienChemin.elementAt(i);
            resu = resu+l.toString()+"\n";
         }
         return resu;
      }
   
   /**
    * Renvoie le chemin dans le format XML dfinit dans <TT>graph.dtd</TT>.
    * @author Claudia
    */
      public String toXML()
      {
         LienChemin l;
         String resu="\t<NomChemin>" + nomChemin + 
         	"</NomChemin>\n\t<Cours>" + cours + "</Cours>\n";
         for (int i=0;i<vectLienChemin.size();i++){
            l=(LienChemin) vectLienChemin.elementAt(i);
            resu = resu + "\t<LienChemin>\n\t\t<NoeudD>" + l.getNoeudD().getIdent() +
            	"</NoeudD>\n\t\t<NoeudA>" + l.getNoeudA().getIdent() +
            	"</NoeudA>\n\t\t<Cours>" + cours + "</Cours>\n\t</LienChemin>\n";
         }
         return resu;
      }
   
    /**
     * retourne l'index du lienChemin reliant les noeud d'ident noeudD et noeudA
     * -1 si il n'existe pas 
     */
      public int indexLien(Noeud noeudD, Noeud noeudA)
      {
         Lien l;
      
         for (int i=0; i<vectLienChemin.size(); i++)
         {
            l = (LienChemin)vectLienChemin.elementAt(i);
         
            if ( (noeudD.equals(l.getNoeudD())) &&
                    (noeudA.equals(l.getNoeudA())) )
               return i;
         }
         return (-1);
      }
   
    /**
     *renvoie vrai si un lien de ce chemin allant de identD a identA existe deja
     */
      public boolean CheminExistentEntreNoeuds(Noeud nD,Noeud nA,LienChemin lien)
      {
         Lien l;
         for (int i=0; i<vectLienChemin.size(); i++)
         {
            l = (Lien) vectLienChemin.elementAt(i);
            if (lien.equals(l))
               return true;
         }
         return false;
      }
   
    /**
     * Ajoute le lien "l" a la liste. Verifie que les noeuds 
     * constituants sont declars.
     * @return 
     *   0 : pas de probleme, <br>
     *   1 : le noeud de depart n'est pas dans la liste,<br>
     *   2 : le noeud d'arrivee n'est pas dans la liste,<br>
     *   3 : les deux noeuds du lien ne sont pas dans la liste,<br>
     *   4 : un lien de ce chemin existe deja.
     */
      public int ajouterLienChemin(LienChemin l)
      {
         if (CheminExistentEntreNoeuds(l.getNoeudD(),l.getNoeudA(),l))
            return 4;
      
         if ( (!(graphePere.noeudAppartient(l.getNoeudD()))) && 
              !(graphePere.noeudAppartient(l.getNoeudA())) ) 
            return(3);
      
         if ( !(graphePere.noeudAppartient(l.getNoeudD())) )  
            return(1);
         if ( !(graphePere.noeudAppartient(l.getNoeudA())) )  
            return(2);
      
         vectLienChemin.addElement(l);
         return 0;
      }
   
    /**
     *  Supprime tous les liens du chemin qui ont pour noeud d'origine ou 
     *  d'arrivee le noeud passe en parametre.
     */   
      public void supprimerCheminAppartientAuNoeud(Noeud n)
      {
         LienChemin l2;
         for (int i=0;i<vectLienChemin.size();i++)
         {
            l2 = (LienChemin) vectLienChemin.elementAt(i);
            if (l2.appartientAuNoeud(n))
            {   
               vectLienChemin.removeElementAt(i);
               i--;
            }
         }
      }
   
    /**
     * Supprime le lien qui contient les memes identificateurs
     * de noeud que l, si il trouve. Sinon ne fait rien
     */
      public void supprimerLienChemin(LienChemin l)
      {
         Lien l2;
         for (int i=0;i<vectLienChemin.size();i++)
         {
            l2 = (LienChemin) vectLienChemin.elementAt(i);
            if (l2.equals(l))
            {
               vectLienChemin.removeElementAt(i);
            }
         }
      }
   
      public void paint (Graphics g)
      {
         LienChemin l;
         for (int i=0;i<vectLienChemin.size();i++){
            l=(LienChemin) vectLienChemin.elementAt(i);
            l.paint(g);
         }
      }
   }
