// *********************************************
// Modificications pour mise en libre :
// Modif C le 10/02/2000 par Marc Dauplait :
// modif methode indexLien pour tenir compte des LienBiDir pour la suppression de liens
// ajout mthode setVecteurChemin pour pallier a l'absence des fonctions d'dition des chemins
// *********************************************

   package carto.graphe;

   import java.util.Vector;
   import java.util.Iterator;
   import java.awt.Graphics;
   import diva.graph.model.*;

/**
 * Classe qui contient toutes les informations du graphes. 
 * Elles sont transcrites ici  
 * partir du fichier XML par la classe CreeGraphe.
 */


   public class Graphe extends BasicGraph
   {
   
    /**
     *  Les informations mta du graphe.
     */    
      public MetaGraphe meta;
   
    /**
     *  Le vecteur qui contient tous les noeuds.
     */    
      protected Vector vecteurNoeud;
    /**
     *  Le vecteur qui contient tous les liens.
     */
      protected Vector vecteurLien; 
    /**
     *  Le vecteur qui contient tous les chemins.
     */
      protected Vector vecteurChemin;
   
      private boolean vecteurNoeudVide;
      private boolean vecteurLienVide;
      private boolean vecteurCheminVide;
   
    /**
     * Constructeur qui initialise un graphe vide, qui peut tre rempli par la suite 
     * des donnes d'un fichier XML, grce  la classe CreeGraphe.
     */
      public Graphe()
      {
         vecteurNoeud=new Vector();
         vecteurLien=new Vector();
         vecteurChemin= new Vector();
      
         vecteurNoeudVide=true;
         vecteurLienVide=true;
         vecteurCheminVide=true;
      
         meta=new MetaGraphe();
      }
   
    /**
     * Constructeur qui initialise un graphe image du graphe pass en paramtre.
     * @param graphe Graphe qui est recopi.
     */
      public Graphe(Graphe graphe)
      {
         vecteurNoeud=new Vector(graphe.vecteurNoeud);
         vecteurLien=new Vector(graphe.vecteurLien);
         vecteurChemin=new Vector(graphe.vecteurChemin);
      
         vecteurNoeudVide=graphe.vecteurNoeudVide;
         vecteurLienVide=graphe.vecteurLienVide;
         vecteurCheminVide=graphe.vecteurCheminVide;
      
         meta=graphe.meta;
      }
   
    /**
     * Construit un graphe vide avec une zone alloue pour placer de noeuds.
     * @param n	Nombre de noeuds  prevoir
     */
      public Graphe(int n) 
      {
         this(null);
         vecteurNoeud=new Vector(n);
      }
   
    /**
     * tablit un chemin dans le graphe avec le nombre du cours choisi.
     */
      public void setChemin(Chemin ch)
      {
         vecteurChemin.addElement(ch);
         vecteurCheminVide=false;
      }
   
    /**
     *  Donne un vecteur contenant tous les noeuds.
     */
      public Vector getVecteurNoeud()
      {
         return vecteurNoeud;
      }
   
    /** 
     * Donne un vecteur contenant tous les liens.
     */
      public Vector getVecteurLien()
      {
         return vecteurLien;
      }
   
    /**
     *  Donne un vecteur contenant tous les chemins.
     */
      public Vector getVecteurChemin()
      {
         return vecteurChemin;
      }

     /**
     *   Dfini le vecteur des chemin du graphe
     *   @param vecteur de chemins a associer au graphe
     */
     //   Temporaire, jusqu' ce que l'dition des chemin soit mise en place
     //   Utilis par carto.editor.EditeurFrame mthode GraphToModel

      public void setVecteurChemin(Vector vc)				// Modif C
      {														// Modif C
         vecteurChemin = vc;								// Modif C
      }														// Modif C
        
    /**
     * Associe au graphe un mtagraphe.
     * @param donnees Metagraphe a associer
     */
      public void ajouterMetaGraphe(MetaGraphe donnees)
      {
         meta=donnees;
      }
   
    /** Ajouter un noeud  la liste des lments. Renvoie 1 si ce n'est pas possible
     *  parce qu'il y a un autre noeud de mme ident.
     */
      public int ajouterNoeud (Noeud n)
      {
         if (!vecteurNoeudVide)
            if (noeudAppartient(n)) 
               return 1;
         vecteurNoeud.addElement(n);
         vecteurNoeudVide = false;
         return 0;
      }
   
    /** Renvoie vrai si le noeud d'ident "ident" appartient dj  la liste. */
      public boolean noeudAppartient(Noeud n)
      {
         Noeud naux;
         for (int i=0; i<vecteurNoeud.size(); i++)
         {
            naux=(Noeud)vecteurNoeud.elementAt(i);
            if ((naux).equals(n))
               return true;
         }
         return false;
      }
   
    /** Retourne l'index du noeud n. -1 si il n'existe pas. 
    */
      public int indexNoeud(Noeud n)
      {// methodes 'indexOf' de la classes 'Vector ??
         for (int i=0; i<vecteurNoeud.size(); i++)
            if (n.equals((Noeud)vecteurNoeud.elementAt(i))) 
               return i;
         return (-1);
      }
   
    /** 
     * Donne l'index d'un noeud.
     * @param ident L'identificateur du noeud
     * @return l'index du noeud d'ident "ident". "null" si il n'esixte pas. 
     */
      public Noeud getNoeud(String ident)
      {
         for (int i=0; i<vecteurNoeud.size(); i++)
         {
            Noeud n = (Noeud)vecteurNoeud.elementAt(i);
            if ( n.getIdent().equals(ident) ) 
               return n;
         }
         return null;
      }
      public Iterator nodes() {
         return vecteurNoeud.iterator();
      }
   
    /** @return l'index du lien reliant les noeuds noeudD et noeudA.
     * -1 si il n'existe pas 
     */
      public int indexLien(Noeud noeudD, Noeud noeudA)
      //FIXME : Cette methode ne reconnait pas des liens bidirectionnels!!!      
      // Reconnaissance des liens BiDir OK : Dauplait Modif C
      {
         Lien l;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l = (Lien)vecteurLien.elementAt(i);
         
            if (l instanceof LienUniDir) {						// Modif C
               if ( (noeudD.equals(l.getNoeudD())) &&
                    (noeudA.equals(l.getNoeudA())) )
                  return i;
            }													// Modif C
            else if (l instanceof LienBiDir) {					// Modif C
               LienUniDir l1 = ((LienBiDir)l).premier();		// Modif C
               LienUniDir l2 = ((LienBiDir)l).deuxieme();		// Modif C
               if  (											// Modif C
                      ( (noeudD.equals(l1.getNoeudD())) &&		// Modif C
                        (noeudA.equals(l1.getNoeudA())) )		// Modif C
                   ||											// Modif C
                      ( (noeudD.equals(l2.getNoeudD())) &&		// Modif C
                        (noeudA.equals(l2.getNoeudA())) )		// Modif C
                    )											// Modif C
                  return i;										// Modif C
            }													// Modif C
         }
         return (-1);
      }
   
   /** @return vrai si un lien allant de identD a identA existe deja */
      public boolean lienExiste(String identD, String identA)
      {
         Lien l;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l = (Lien) vecteurLien.elementAt(i);
            if ((l.getNoeudD().getIdent().equals(identD)) 
                && (l.getNoeudA().getIdent().equals(identA)))
               return(true);
         }
         return(false); 
      } 
   
   /** @return l'index du lien oppose du lien "l" si il apppartient dj 
   *  la liste. -1 sinon 
   * @author Eric
   */
   
      public int lienAppartientDeja(Lien l)
      {
         Lien l2;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l2 = (Lien) vecteurLien.elementAt(i);
            if ((l2.getNoeudD().equals(l.getNoeudA())) && (l2.getNoeudA().equals(l.getNoeudD()))) 
               return i;
         }
         return (-1);
      }
   
   
   /** @return l'index du lien oppose du lien "l" si il apppartient dj 
   *  la liste. -1 sinon   methode utilisee par LIENUNIDIR
   */
      public int lienOpposeAppartient(LienUniDir l)
      {
         Lien l2;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l2 = (Lien) vecteurLien.elementAt(i);
            if ( l2 instanceof LienUniDir )
            {
               if ((l2.getNoeudD().equals(l.getNoeudA())) && 
                      (l2.getNoeudA().equals(l.getNoeudD())))
                  return i;
            }
         }
         return (-1);
      }
   
   /** @return vrai si un lien allant de identD a identA existe deja */
      public boolean lienEgal(Lien newlien)
      {
         Lien l;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l = (Lien) vecteurLien.elementAt(i);
         
            if (l.equals(newlien)) 
               return (true);
         }
         return(false); 
      }
   
    /** @return Vrai si lien est bien placee entre le noeud de depart nd et le noeud d'arrivee nA. Faux sinon.
     */   
      public boolean lienExistentEntreNoeuds(Noeud nD,Noeud nA,Lien lien)
      {
         Lien l;
         LienBiDir laux;
         LienUniDir laux2;
         for (int i=0; i<vecteurLien.size(); i++)
         {
            l = (Lien) vecteurLien.elementAt(i);
            if ( l instanceof LienUniDir )
            {
               laux2 = (LienUniDir)l;
            
               if( ( lien instanceof LienUniDir ) &&
                      (laux2.directionEquals((LienUniDir)lien)) )
                  return true;
               else if ( lien instanceof LienBiDir )
               {
                  laux = (LienBiDir)lien;
               
                  if  (laux.premier().directionEquals(laux2) ||
                       laux.deuxieme().directionEquals(laux2)) 
                     return true;
               }
            }
            else if( l instanceof LienBiDir )
            {
               laux = (LienBiDir)l;
            
               if ( lien instanceof LienUniDir )
               {
                  laux2 = (LienUniDir)lien;
                  if  (laux.premier().directionEquals(laux2) ||
                       laux.deuxieme().directionEquals(laux2))
                     return true;
               }
               else if ( ( lien instanceof LienBiDir ) &&
                         laux.noeudsEquals((LienBiDir)lien) )
                  return true;
            }
         }
         return false;
      }
   
    /**
     *  @return Un lien bidirectionel construit a partir de deux lien unidirectionnels
     * @param l1 le premier lien unidirsctionnel
     * @param l2 le deuxieme lien unidirectionnel
     */
      public LienBiDir toLienBi(LienUniDir l1,LienUniDir l2)
      {  //deja teste que l1 et l2 ont les memes noeuds
         return new LienBiDir(l1,l2);
      }	 
   
    /**
     *  Ajoute au graphe un lien bidirectionnel, cree a partir de deux liens unidirectionnels qui existent deja dans le graphe
     * @param l lien bidirectionnel cree a partie de deux liens monodirectionnel du graphe
     * @return  0 si l'operation est reussie, un nombre suprieur sinon.
     */
      public int ajouterLienBi(LienBiDir l)
      {  
         if (lienExistentEntreNoeuds(l.premier().getNoeudD(),
                                     l.premier().getNoeudA(),l))
            return(4);
         if ( !(noeudAppartient(l.premier().getNoeudD())) && 
              !(noeudAppartient(l.premier().getNoeudA())) )
            return(3);
         if ( !(noeudAppartient(l.premier().getNoeudD())) ||
              !(noeudAppartient(l.deuxieme().getNoeudD())))
            return(1);
         if ( !(noeudAppartient(l.premier().getNoeudA())) ||
              !(noeudAppartient(l.deuxieme().getNoeudA())))
            return(2);
         vecteurLien.addElement(l);
         vecteurLienVide = false;
         return (0);
      }
   
   /** Ajoute le lien "l" a la liste. Verifie que les noeuds 
   * constituants sont declares
   * Si le lien oppose existe deja, modifie ce lien. Renvoie :
   *   0 : pas de probleme
   *   1 : le noeud de depart n'est pas dans la liste 
   *   2 : le noeud d'arrivee n'est pas dans la liste
   *   3 : les deux noeuds du lien ne sont pas dans la liste
   *   4 : le lien existe deja
   */
      public int ajouterLienUni(LienUniDir l)
      {
         int index;
         LienBiDir lienbi;
         LienUniDir lienuni;
      
      
         if (lienExistentEntreNoeuds(l.getNoeudD(),l.getNoeudA(),l))
            return 4; 
      
         if ( (!(noeudAppartient(l.getNoeudD()))) && 
              !(noeudAppartient(l.getNoeudA())) ) 
            return(3);
      
         if ( !(noeudAppartient(l.getNoeudD())) ) 
            return(1);
      
         if ( !(noeudAppartient(l.getNoeudA())) ) 
            return(2);
      
         index = lienOpposeAppartient(l);
         if  (index != -1) 
         {
            lienuni = (LienUniDir)vecteurLien.elementAt(index); 
            if  (l.getNoeudD().equals(lienuni.getNoeudA()) && 
                 l.getNoeudA().equals(lienuni.getNoeudD())) 
            {
               lienbi = toLienBi(l,lienuni);
               vecteurLien.setElementAt(lienbi,index);
            }
         }
         else
         {
            vecteurLien.addElement(l);
            vecteurLienVide = false;
         }
         return 0;
      }
   
   /** Supprime le lien qui contient les mmes identificateurs
   * de noeud que l (, si il trouve. Sinon ne fait rien.)
   */
      public void supprimerLien(Lien l)
      {
         Lien l2;
         for (int i=0;i<vecteurLien.size();i++)
         {
            l2 = (Lien)vecteurLien.elementAt(i);
            if (l2.equals(l))
            {
               vecteurLien.removeElementAt(i);
               if (vecteurLien.size()==0) vecteurLienVide=true;
               return;
            }
         }
      }
   
   /** 
    * Supprime le noeud n avec ses liens et lienChemins.
    */
      public void supprimerNoeud(Noeud n)
      {
         Noeud n2;
         Lien l;
         Chemin ch;
         
         for (int i=0;i<vecteurNoeud.size();i++)
         {
            n2 = (Noeud) vecteurNoeud.elementAt(i);
            if (n2.equals(n))
            {
            // on supprime les liens a qui il appartenait
               for (int j=0;j<vecteurLien.size();j++)
               {
                  l= (Lien) vecteurLien.elementAt(j);
                  if (l.appartientAuNoeud(n))
                  { 
                     vecteurLien.removeElementAt(j);
                     if (vecteurLien.size()==0) vecteurLienVide=true;
                     j--;     // parce que le vecteur s'est "tase" 
                  }
               }
               for (int j=0;j<vecteurChemin.size();j++)
               {
                  ch= (Chemin) vecteurChemin.elementAt(j);
                  ch.supprimerCheminAppartientAuNoeud(n);
               }
            
            //   on le supprime du vecteur des  noeud 
               vecteurNoeud.removeElementAt(i);
               if (vecteurNoeud.size()==0) vecteurNoeudVide=true;
            }   
         } 
      } 
   
   
   /**
    * Change le noeud partout ou il apparait dans le graphe.
    * @param noeud l'ancien noeud qui a des liens et lien chemin dans le graphe
    * @param noeudAModifier le noeud avec de nouvelles valeurs
    * @author Claudia
    */	
      public void modifierNoeud( Noeud noeud, Noeud noeudAModifier)
      { 
      // => supprime l'ancien, ajoute le nouveau dans le graphe
         int index = indexNoeud(noeud);
         if ( index != -1 ) {
            vecteurNoeud.removeElementAt(index);
            vecteurNoeud.add(index, noeudAModifier);
         }
         else 
         //l'information du graphe auquel il appartient
         noeudAModifier.setParent(noeud.getParent());
         noeudAModifier.setCoordX(noeud.getCoordX());
         noeudAModifier.setCoordY(noeud.getCoordY());
         //changer les liens
         if (!vecteurLienVide){
            for (int i=0;i<vecteurLien.size();i++){
               Lien l=(Lien) vecteurLien.elementAt(i);
               if ( l.appartientAuNoeud(noeud) ) 
                  changeNoeudDansLien(l, noeud, noeudAModifier);
            
            }
         }
         if (!vecteurCheminVide){
            for (int i=0;i<vecteurChemin.size();i++){
               Chemin ch = (Chemin) vecteurChemin.elementAt(i);
               Vector vectLienCh = (Vector) ch.vectLienChemin;
               if (!vectLienCh.equals(null)){
                  for (int j=0;j<vectLienCh.size();j++){
                     LienChemin lc=(LienChemin) vectLienCh.elementAt(j);
                     if ( lc.appartientAuNoeud(noeud) ) 
                        changeNoeudDansLien(lc, noeud, noeudAModifier);
                  }
               } 
            } 
         }
      }
   
      protected void changeNoeudDansLien ( Lien lien, Noeud noeud, Noeud noeudAModifier)
      {
         if ( noeud.equals(lien.getNoeudD()) ) lien.setNoeudD(noeudAModifier);
         else if ( noeud.equals(lien.getNoeudA()) ) lien.setNoeudA(noeudAModifier);
      }
   
      public String toString(){
      
         String resu="";
         Noeud n;
         Lien l;
         Chemin c;
         resu = resu + meta.toString()+"\n";
         if (!vecteurNoeudVide){
            for (int i=0;i<vecteurNoeud.size();i++){
               n=(Noeud) vecteurNoeud.elementAt(i);
               resu = resu+n.toString()+"\n";
            }
         }
         if (!vecteurLienVide){
            for (int i=0;i<vecteurLien.size();i++){
               l=(Lien) vecteurLien.elementAt(i);
               resu = resu+l.toString()+"\n";
            }
         }
         if (!vecteurCheminVide){
            for (int i=0;i<vecteurChemin.size();i++){
               c=(Chemin) vecteurChemin.elementAt(i);
               resu = resu+c.toString()+"\n";
            }	
         }
         return resu;
      }
   
    /**
     *  Renvoie le nom du fichier XML correspondant au graphe de niveau suprieur.
     */   
      public String getPere()
      {
         return meta.getPere();
      }
   
    /**
     * Associe  chaque lment du graphe, du package carto.graphe, un dessin du package 
     * carto.dessin. Ce choix est fait en fonction du style. Dans notre version, il y a deux styles, 
     * celui qui sert  faire des index bien compacts (symbolis par le chiffre 2), et celui qui est 
     * utilis par dfaut (chiffre 1).
     * @param style Entier qui symbolise le style dans lequel les dessins vont tre choisis.
     * @author Fred
     */   
      public void setStyle(int style)
      {
         Noeud n;
         Lien l;
         LienChemin lc;
         Chemin ch;
         Vector vectLienCh;
      
         if (!vecteurNoeudVide){
            for (int i=0;i<vecteurNoeud.size();i++){
               n=(Noeud) vecteurNoeud.elementAt(i);
               n.setStyle(style);
            }
         }
      }
   
   /**
    *  Mthode qui dessine un graphe en affichant une difficult de 3
    *  par defaut, et le premier chemin.
    */   
      public void paint(Graphics g)
      {
         dessine(g,3,0);
      }
   
   /** Sait dessiner un graphe. 
    * Parcourt les vecteurs vecteurNoeud, vecteurLien, vecteurChemin 
    * et appelle la mthode 'paint(g)' pour chaque lment des vecteurs.
    * @param difficulte Determine la difficult maximale des noeuds a afficher
    * @param numChemin Determine le numero du chemin a afficher
    */   
      public void dessine(Graphics g,int difficulte,int numChemin)
      {
         Noeud n;
         Lien l;
         LienChemin lc;
         Chemin ch;
         Vector vectLienCh;
      
         if (!vecteurNoeudVide){
            for (int i=0;i<vecteurNoeud.size();i++){
               n=(Noeud) vecteurNoeud.elementAt(i);
               if (!(n.getDifficulte()>difficulte))
                  n.paint(g);
            }
         }
         if (!vecteurLienVide){
            for (int i=0;i<vecteurLien.size();i++){
               l=(Lien) vecteurLien.elementAt(i);
               if (!((l.getNoeudA().getDifficulte()>difficulte)||(l.getNoeudD().getDifficulte()>difficulte)))
                  l.paint(g);
            }
         }
         if (!vecteurCheminVide){
            ch = (Chemin) vecteurChemin.elementAt(numChemin);
            vectLienCh = (Vector) ch.vectLienChemin;
            if (!vectLienCh.equals(null)){
               for (int j=0;j<vectLienCh.size();j++){
                  lc=(LienChemin) vectLienCh.elementAt(j);
                  if (!((lc.getNoeudA().getDifficulte()>difficulte)||(lc.getNoeudD().getDifficulte()>difficulte))) 
                     lc.paint(g);
               }
            }  
         }
         meta.paint(g);
      }
   
   ///////////////////////////////////////////////////////////////////////////////////
   ////// Methodes ajoutee pour la colaboration de cette classe avec "diva.*"  ///////
   
    /**
     * Construit un graphe vide avec le userObject spcifi.
     */
      public Graphe(Object userObject) 
      
      {
         this();
         setSemanticObject(userObject);
      }
   
   
      public void add(BasicNode n) {
         Noeud noeud = (Noeud) n;
         ajouterNoeud(noeud);
         noeud.setParent(this);
      }
   
   
      public boolean contains(Node n) {
      
         return noeudAppartient((Noeud) n);
      }
   
    /**
     *  Renvoie l'index d'un noeud.
     *  @param n Le noeud dont on veut connaitre l'index.
     *  @return L'index du noeud
     */   
      public int getIndex(Node n) {
      
         return vecteurNoeud.indexOf((Noeud)n);
      }
   
    /**
     *  Retourne un noeud du graphe.
     *  @param i index du noeud dans le graphe
     */
   
      public Node getNode(int i) {
      
         return (Node)vecteurNoeud.elementAt(i);
      }
   
   
      public void addEdge(Edge e)
      
      {
         ajouterLienUni((LienUniDir) e);
      }
   
      /** 
       * La methode 'supprimerNoeud(Noeud n)'. 
       */   
      public void delete(BasicNode n) {
      
         //n.getInEdgeSet().clear();
         //n.getOutEdgeSet().clear();
         supprimerNoeud((Noeud) n);
      }
   
      /** 
       * Le noeud n'est que supprime dans le vecteur 'vecteurNoeud'.
       */   
      public void remove(BasicNode n) {
      
         Noeud noeud = (Noeud) n;
         int index = vecteurNoeud.indexOf(noeud);
         vecteurNoeud.remove(index);
      }
   
      /**
       * Tous les noeuds du vecteur sont supprimes.
       */   
      public void removeAll() {
      
         vecteurNoeud.clear();
      }
   
      public void clear() {
      
         while(getNodeCount() > 0) {
            Noeud n = (Noeud)getNode(0);
            delete(n);
         }
      }
   
      /** 
       * @return La taille du 'vecteurNoeud'.
       */
      public int getNodeCount() {
      
         return vecteurNoeud.size();
      }	
   }



