   package carto.graphe;

   import carto.dessin.*;
   import java.awt.*;
   import diva.graph.model.*;

/**
 * Un Lien reprsente un segment qui relie un noeud de Dpart et un noeud 
 * d'Arrive. Chaque implmentation de cette classe correspond  un type de lien.
 */
   public abstract class Lien extends BasicEdge
   {
      protected Noeud noeudD;
      protected Noeud noeudA;
   
      public Dessin dessin; 
   
   /**
    * Cree un Lien entre deux noeuds nD et nA avec une epaisseur egale a un 
    * et une trace continue par defaut.
    * @param nD noeud de Depart
    * @param nA noeud d'Arrivee
    */
      public Lien(Noeud nD, Noeud nA) {
      // !!! l'ordre des noeuds est change pour l'adapter aux classes 'diva.graph.model.*'
         this(null, nA, nD);
      }
   
      public String toString()
      {
         return (getClass().toString()+" noeudD: "+noeudD.getIdent()+" noeudA: "
            +noeudA.getIdent());
      }
   
   /**
    * Renvoie le noeud de Dpart du Lien.
    */ 
      public Noeud getNoeudD(){
         return noeudD; }
   
   /**
    * Renvoie le noeud d'Arrive du Lien.
    */
      public Noeud getNoeudA(){
         return noeudA; }
   
   /**
    * tablit le noeud de Depart du Lien.
    * @param nD noeud de Depart
    */
      public void  setNoeudD(Noeud nD){ 
         noeudD = nD; }
   
   /**
    * tablit le noeud d'Arrivee du Lien.
    * @param nA noeud d'Arrivee
    */
      public void  setNoeudA(Noeud nA){ 
         noeudA = nA; }  
   
   /**
    * Renvoie true si le Lien est egale a celui qui est passe comme parametre
    * false en cas contraire.
    * @param l un autre Lien
    */
      public boolean equals(Lien l)
      {
         return ((noeudD.equals(l.noeudD) && noeudA.equals(l.noeudA)) ||
                    (noeudA.equals(l.noeudD) && noeudD.equals(l.noeudA)));
      }   
   
   /** 
    * Renvoie vrai si le lien appartient au noeud pass en paramtre.
    */
      public boolean appartientAuNoeud(Noeud n)
      {
         return (noeudA.equals(n) || noeudD.equals(n));
      }
   
   /**
    * En fonction du type de lien, on aura des paramtres graphiques diffrants. 
    * Un type de lien correspond  une implmentation de l'interface lien. 
    */
      public abstract float getDefautEpaisseur();
      
   /**
    * En fonction du type de lien, on aura des paramtres graphiques diffrants. 
    * Un type de lien correspond  une implmentation de l'interface lien.
    */
      public abstract Color getDefautCouleur();
      
   /**
    * Renvoie le type du lien. Un type de lien correspond  une implmentation de 
    * l'interface lien. 
    */
      public abstract String getType();
   
   /**
    * Dessine le lien.
    * @param g environement graphique 
    */ 
      public abstract void paint(Graphics g);
   
   ///////////////////////////////////////////////////////////////////////////////////
   ////// Methodes ajoutee pour la colaboration de cette classe avec "diva.*"  ///////
   
      public Lien(Object userObject, Noeud nA, Noeud nD)
      
      {
         setSemanticObject(userObject);
         attach(nA, nD);
      }
   
    /**
     * Create a new edge with no tail or head.
     */              
      public Lien() {
      
         this(null);
      }
   
    /**
     * Create a new edge with the specified user object
     * but no tail or head.
     */              
      public Lien(Object userObject) {
      
         this(userObject, null, null);
      }
   
    /**
     * Create a new edge with the specified tail/head.
     */              
      public Lien(BasicNode tail, BasicNode head) {
      
         this(null, (Noeud) head, (Noeud) tail);
      }
   
   
      public void attach(Noeud tail, Noeud head) {
      
         setNoeudA(tail);
         setNoeudD(head);
      }
   
   
      public void detach() {
      
         setNoeudD(null);
         setNoeudA(null);
      }
   
   
      public Node getHead() { 
      
         return (Node) noeudA; }
   
   
      public Node getTail() { 
      
         return (Node) noeudD; }
   
   
   
      public void setHead(Noeud n) {
      
         Noeud prevHead = (Noeud)getHead();
         if(prevHead != null) {
            prevHead.getInEdgeSet().remove(this);
         }
      
         noeudA = n;
         if(noeudA != null) {
            noeudA.getInEdgeSet().add(this);
         }
      }
   
   
      public void setTail(Noeud n) {
      
         Noeud prevTail = (Noeud)getTail();
         if(prevTail != null) {
            prevTail.getOutEdgeSet().remove(this);
         }
      
         noeudD = n;
         if(noeudD != null) {
            noeudD.getOutEdgeSet().add(this);
         }
      }
   }
