   package carto.graphe;

   import carto.dessin.*;
   import java.awt.*;
   import java.awt.geom.*;

/**
 * LienUniDir reprsente un Lien unidirectionnel
 */
   public class LienUniDir extends Lien
   {
      protected int difficulte=2;
      protected String type;
   
   /**
   * Cree un Lien unidirectionnel avec une difficult gale  deux par dfaut.
   * @param nD noeud de depart
   * @param nA noeud d'arrivee
   */
      public LienUniDir(Noeud nD, Noeud nA)
      {
         super(nD,nA);
         dessin = new Ligne(this);
         dessin.setCouleur(getDefautCouleur());
      }
   
      public LienUniDir (){
         this(null, null);
      }
   
      public String toXML ()
      { // @author Claudia
         String resu = "\t<noeudD>" + noeudD.getIdent() + "</noeudD>\n\t<noeudA>" + 
         noeudA.getIdent() + "</noeudA>\n\t<TypeL>";
         if (this instanceof Lien_CO )       resu = resu + "CO";
         else if (this instanceof Lien_CA )  resu = resu + "CA";
         else if (this instanceof Lien_D )   resu = resu + "D";
         else if (this instanceof Lien_PO )  resu = resu + "PO";
         else if (this instanceof Lien_SANS) resu = resu + "SANS";
         else if (this instanceof Lien_T )   resu = resu + "T";
         resu = resu + "</TypeL>\n\t<Diff>" + difficulte+ "</Diff>\n";
         return resu;
      }
   
   /**
    * Reenvoie le type du Lien Unidirectionel.
    */
      public String getType()
      {
         return type;
      }
   
   /**
   * Renvoie le niveau de  difficult du lien
   * @return le niveau de difficult du lien
   */
      public int getDifficulte()
      {
         return difficulte;
      }
   
      public void setDifficulte(int diff)
      {difficulte=diff;}
   
   /**
    * Renvoie l'epaisseur pour dessiner le lien.
    * @return l'epaisseur pour tous les liens unidirectionnels
    * l'epaisseur peut etre: 1, 2, ou 3
    */
      public float getDefautEpaisseur()
      {
         return 1.0f;
      }
   
   /**
    * Renvoie la couleur pour dessiner le lien.
    * @return la couleur pour tous les liens unidirectionnels
    */
      public Color getDefautCouleur()
      {
         return Color.black;
      }
   
   /** 
    * Reenvoie true si deux liens unidirectionels sont du mme type.
    */
      public boolean typeEqualsEntreNoeuds(LienUniDir l)
      {
         if (directionEquals(l))
         {
            return (type.equals(l.getType()));
         }
         return false;
      } 
   
   /**
    * Reenvoie true si le lienunidirectionel parametre a le noeud depart et
    * le noeud d'arrive egales  ce noeud.
    */         
      public boolean directionEquals(LienUniDir l)
      {
         return (noeudD.equals(l.noeudD) && noeudA.equals(l.noeudA));
      }
   
   
   /**
    * Reenvoie true si un lien unidirectionel est gal e ce lien unidirectionel
    * cela veux dire que les deux ont le mme type, les mmes noeuds dpart et
    * d'arrive et la mme dificult.
    */
      public boolean equals(Lien l)
      {
         LienUniDir lu;
      
         if ( l instanceof LienUniDir )
         {
            lu = (LienUniDir) l;
            if (typeEqualsEntreNoeuds(lu))
               return (getDifficulte() ==lu.getDifficulte());
         }
         return false;
      }
   
   
   /**
    * Dessine un lien unidirectionnel.
    * @param g l'environnement graphique
    */
      public void paint(Graphics g)
      { 
         dessin.etablirDessin();
         dessin.paint(g);
      }
   }


