package carto.graphe;

import carto.dessin.*;

import java.util.GregorianCalendar;
import java.awt.Graphics;

/**
* La classe enregistre toutes les valeurs mtas du graphe. <br>
* Note that in class java.util.GregorianCalendar the value used to 
* set the MONTH is 0-based. e.g., 0 for January.
*/

public class MetaGraphe{
	
	protected String titre, createur, motsCles, version, pere;
	protected GregorianCalendar date;
	protected int niveau;
	
	public Dessin dessin;
	
	public MetaGraphe()
	{
		titre="";
		createur="";
		motsCles="";
		version="";
		date = new GregorianCalendar();
		dessin = new MetaTexte(this);
	}
	
    /**
	*  Construit un metagraphe avec les informations donnes en paramtre.
	*/   
	public MetaGraphe(String unTitre, String unCreateur, String desMots, 
		String laVersion, String lePere, int niv)
	{
		titre=unTitre;
		createur=unCreateur;
		motsCles=desMots;
		version=laVersion;
		pere=lePere;
		niveau=niv;
		date = new GregorianCalendar();
		dessin = new MetaTexte(this);
	}
	
	
	public void setMetaGraphe(String unTitre, String unCreateur, String desMots, 
		String laVersion, String lePere, int niv)
	{
		titre=unTitre;
		createur=unCreateur;
		motsCles=desMots;
		version=laVersion;
		pere=lePere;
		niveau=niv;
	}
	
	
    /**
	*  Choisit le titre du metagraphe.
	*  @param unTirtre Nouvelle valeur du titre
	*/   
	public void setTitre(String unTitre)
	{ 
		titre=unTitre;
	}
	
    /**
	*  @return Le titre du metagraphe
	*/ 
	public String getTitre()
	{
		return titre;
	}  
	
    /**
	* Choisit le graphe de niveau suprieur du graphe auquel le 
	* mtagraphe est associ. 
	* @param lePere Nom du fichier XML dcrivant le graphe de niveau suprieur
	*/
	public void setPere(String lePere)
	{
		pere=lePere;
	} 
	
    /**
	* @return Le nom du fichier XML contenant le graphe de niveau suprieur 
	* du graphe auquel le mtagraphe est associ.
	*/ 
	public String getPere()
	{
		return pere;
	}   
	
	/**
	*  Choisit le niveau du graphe auquel le metagraphe est associ.
	*  @param niv Valeur du nouveau niveau.
	*/   
	public void setNiveau(int niv)
	{
		niveau=niv;
	} 
	
    /**
	*  @return La valeur du niveau du graphe auquel le metagraphe est associe.
	*/
	public int getNiveau ()
	{ 
		return niveau;
	}
	
    /**
	*  Choisit le createur du graphe.
	*  @param unCreateur Nom du createur.
	*/
	public void setCreateur(String unCreateur)
	{ 
		createur=unCreateur;
	}  
	
    /**
	*  Renvoie le nom du createur du graphe auquel le metagraphe est associe.
	*/
	public String getCreateur ()
	{
		return createur;
	} 
	
    /**
	*  Choisit les mots clef du graphe auquel le metagraphe est associe.
	*  @param desMots Les mots clef choisis.
	*/
	public void setMotsCles(String desMots)
	{ 
		motsCles=desMots;
	}  
	
    /** Renvoie les mots clef du graphe auquel le metagraphe est associe
	*/
	public String getMotsCles()
	{
		return motsCles;
	} 
	
    /**
	*  Choisit la version du graphe auquel le metagraphe est associe.
	*  @param laVersion Nouvelle version du graphe.
	*/
	public void setVersion(String laVersion)
	{
		version=laVersion;
	}
	
    /**
	*  Renvoit la version du graphe auquel le metagraphe est associe.
	*  @return la version du graphe
	*/
	public String getVersion()
	{
		return version;
	}
	
    /**
	*  Choisit la date de cration du graphe auquel le mtagraphe est associ. 
	*  @param year Annee de ralisation du graphe.
	*  @param month Mois de cration du graphe.
	*  @param day Jour de cration du graphe.
	*/   
	public void setDateCreation(int year, int month, int day)
	{
		GregorianCalendar calendrier = new GregorianCalendar();  
		calendrier.set(calendrier.YEAR,year);
		calendrier.set(calendrier.MONTH,month-1);
		calendrier.set(calendrier.DAY_OF_MONTH,day);
		date=calendrier;
	}
	
    /**
	*  Choisit la date de cration du graphe auquel le mtagraphe est associ. 
	*  @param date Date prcise.
	*/   
	public void setDateCreation(GregorianCalendar date)
	{
		this.date = date;
	}
	
    /**
	*  @return Une chane de caractres reprsentant la date de creation du graphe.
	*/
	public String getDateString()
	{
		return date.get(date.DAY_OF_MONTH) + "/" + (date.get(date.MONTH)+1) + 
            "/" +  date.get(date.YEAR);
	}
	
    /**
	*  @return La date de creation du graphe.
	*/
	public GregorianCalendar getDateCreation()
		
	{
		return date;
	}
	
	/**
	* Donne la reprsentation XML de MetaGraphe.
	*/              
	public String toXML()   
	{
		GregorianCalendar calendrier = new GregorianCalendar();
		String resu = "\t<Titre>" + titre + "</Titre>\n\t<Createur>" + 
			createur + "</Createur>\n\t<MotsCles>" + motsCles +
			"</MotsCles>\n\t<Date>\n\t\t<An>" + calendrier.get(calendrier.YEAR) +
			"</An>\n\t\t<Mois>" + (calendrier.get(calendrier.MONTH)+1) +
			"</Mois>\n\t\t<Jour>" + calendrier.get(calendrier.DAY_OF_MONTH) + 
			"</Jour>\n\t</Date>\n\t<Version>" + version +"</Version>\n\t<Niveau>"
			+ niveau + "</Niveau>\n\t<Pere>" + pere + "</Pere>\n";
		return resu;
	}
	
	/**
	* Donne la reprsentation String de MetaGraphe.
	*/
	
	public String toString() 
	{   
		return("Titre: "+titre+" - Createur: "+createur+
			"\nMots cles: "+motsCles+"\nVersion: "+version+
			" - Date: "+ getDateString() +
			"\nNiveau: "+niveau+" Pere: "+pere+"\n");
	}
	
	public void paint (Graphics g)
	{
		dessin.etablirDessin();
		dessin.paint(g);
	}
	
	public static void main (String args[]){
		MetaGraphe meta = new MetaGraphe();
		System.out.println(meta.toString());
		meta.setTitre("La duplication et le test d'galite entre objets");
		meta.setCreateur("Toto");
		meta.setDateCreation(2000,12,31);
		System.out.println(meta.toString());
		System.out.println(meta.toXML());
	}
}
