   package carto.graphe;

   import carto.dessin.*;
   import java.awt.*;
   import java.net.URL;   
   import diva.graph.model.*;

/**
 * Permet de stocker les informations relatives  un noeud du graphe. 
 * Reprsente une partie d'un cours ou d'un document.
 * La classe est abtraite, pour tre sur qu'elle n'est pas instancie.
 */
   public abstract class Noeud extends diva.graph.model.BasicNode
   
   {
     /**
      * Le graphe auquel ce noeud appartient.
      */
      protected Graphe parent = null; 
   
      protected String ident, nom, symbole, support;
      protected int volume, difficulte, x, y;
      /** 
       * L'URL de la page Web contenant le cours. 
       */
      protected URL url;
      /** 
       * Le fichier xml du niveau infrieur.
       */
      protected String graphe;
      public Dessin dessin;
   
      /**
       * On se cre un noeud avec les caractristiques d'un cours par dfaut.
       */
      public Noeud()
      {
         ident = "";
         nom = "";
         symbole = "";
         support = "HTML";
         url = null;
         volume = 2;
         difficulte = 2;
         x = 40;
         y = 40;
         graphe = ""; 
      }
   
      public void setValeurs(String ident, String nom, String symbole, 
                             String support, int volume, int difficulte, 
                             String graphe, String url){
         this.ident = ident;
         this.nom = nom;
         this.symbole = symbole;
         this.support = support;
         this.volume = volume;
         this.difficulte = difficulte;
         this.graphe = graphe;
         setURL(url);
      }
   
    /**
     * Donne un String contenant les attributs du noeud.
     * @author Claudia
     */
      public String toString ()
      {  
         String resu = "";
         resu = resu +"Ident: "+ident+" Nom: "+nom+" Symbole: "+symbole+" Support: "+support;
         if (url != null) resu = resu + "\nURL: "+url.toString();
         resu = resu + "\nVolume: "+volume+" Difficulte: "+difficulte+
                " x: "+x+" y: "+y+" grahe: "+graphe; //+" parent: "+parent.meta.getTitre();
         return resu;
      }
   
    /**
     * Donne le format XML (selon le DTD "graphe.dtd") du noeud.
     * @author Claudia
     */   
      public String toXML ()
      { 
         String resu = "\t<TypeN>";
         if ( this instanceof Noeud_M) resu = resu + "M";
         else if ( this instanceof Noeud_TD) resu = resu + "TD";
         else if ( this instanceof Noeud_TP) resu = resu + "TP";
         resu = resu + "</TypeN>\n\t<Ident>" +ident+ "</Ident>\n\t<Nom>"+nom+
                "</Nom>\n\t<Symbole>"+symbole+
                "</Symbole>\n\t<Support>"+support+"</Support>\n\t<URL>";
         if (url != null) resu = resu +url.toString();
         resu = resu + "</URL>\n\t<Volume>"+volume+"</Volume>\n\t<Difficulte>"
                +difficulte+"</Difficulte>\n\t<CoordX>"+x+"</CoordX>\n\t<CoordY>"
                +y+"</CoordY>\n\t<Zoom>"+graphe+"</Zoom>\n"; 
         return resu;
      }
   
   ///////////////////////////////////////////////////////////////////////////////////
   ////// les methodes pour 'set' et 'get' de toutes les valeurs du noeud  ///////////   
   
   /**
    * Donne le graphe (le fichier du graphe) infrieur qui dscrit ce noeud.
    */  
      public String zoomer()
      {
         return graphe;
      }
   
      public void setGraphe (String str)
      {graphe=str;}
   
      public Graph getParent() {
         return (Graph)parent;
      }
      public void setParent(Graph g) {
         parent = (Graphe)g;
      }
   
   /**
    * Donne l'identificateur du noeud.
    */
      public String getIdent(){
         return ident;}
   
   /**
    * tablit l'identificateur du noeud.
    */
      public void setIdent(String newident)
      {
         ident = newident;
      }
   
   
   /**
    * Donne le nom du noeud.
    */
      public String getNom(){
         return nom;}
   
   /**
    * tablit le nom du noeud.
    */
      public void setNom(String newnom)
      {
         nom = newnom;
      }
   
   /**
    * Donne le type de support pour suivre une partie d'un cours.
    */
      public String getSupport(){
         return support;}
   
   /**
    * tablit le type de support utilise.
    */
      public void setSupport(String newsupport)
      {
         support = newsupport;
      }
   
   /**
    * Donne le symbole qui est affich  l'interieur du noeud.
    */
      public String getSymbole(){
         return symbole;}
   
   /**
    * tablit le symbole du noeud.
    */
      public void setSymbole(String newsymbole)
      {
         symbole = newsymbole;
      }
   
   /**
    * Donne l'adresse URL qui contient une partie d'un cours.
    */
      public URL getURL(){
         return url;}
   
   /**
    * tablit l'adresse URL qui contient une partie d'un cours.
    */
      public void setURL(String newURL)
      {
         try{
            url = new URL (newURL);
         }
            catch(Exception e) {
               System.out.println("Erreur avec URL : "+url+"\nURL du noeud set to null!"); 
               this.url = null;}
      
      }
   
   /**
    * Donne le volume horaire pour suivre une partie d'un cours.
    */
      public int getVolumeHoraire(){
         return volume;}
   
   /**
    * tablit le volume horaire pour suivre une partie d'un cours.
    */
      public void setVolumeHoraire(int newvolhoraire)
      {
         volume = newvolhoraire;
      }
   
   /**
    * Donne l'abscisse de la position du noeud.
    */
      public int getCoordX(){
         return x;}
   
   /**
    * tablit l'abscisse de la position du noeud.
    */
      public void setCoordX(int newx)
      {
         x = newx;
      }
   
   /**
    * Donne l'ordonne de la position du noeud.
   */
      public int getCoordY(){
         return y;}
   
   /**
    * tablit l'ordonne de la position du noeud.
    */
      public void setCoordY(int newy)
      {
         y = newy;
      }
   
   /**
    * Donne le niveau de difficult (1,2,3) pour suivre une partie d'un cours.
    */
      public int getDifficulte()
      {
         return difficulte;
      }
   
    /**
     * tablit une nouvelle valeur de difficult
   .     * @param newDiff la nouvelle valeur de difficult
     */ 
      public void setDifficulte(int newDiff)
      {
         difficulte = newDiff;
      
      }
   ///////////////////////////////////////////////////////////////////////////////////
   ////// Fin des methodes 'get' 'set'  //////////////////////////////////////////////
   
    /**
     * Renvoie true si le noeud d'entree est egal au meme
     * @param n le noeud a tester
     */
      public boolean equals(Noeud n)
      {
         if ((ident.equals(n.ident))&&
                (nom.equals(n.nom))&&
                (support.equals(n.support))&&
                //(url.equals(n.url))&&
                (volume == n.volume)&&
                (difficulte == n.difficulte) )
            return true;
      
         return false;
      }
   
      public String getType()
      {
         return ("Noeud");
      }
   
      public abstract void setStyle(int style); 
      public abstract Color getDefautCouleur();
   
   
    /**
     * Renvoie true si les coordones i,j apartiennent a l'intrieur du noeud.
     * @param i l'abscise 
     * @param j l'ordonee
     */  
      public boolean inside (int i, int j)
      {
         int rayon = dessin.getRayon();
         return ( ((x-i)*(x-i)+(y-j)*(y-j)) <= (rayon*rayon) );
      }
   
    /**
     * Dessine un noeud avec une forme et une couleur precisees.
     * @param g l'environement graphique
     */
      public void paint(Graphics g)
      {
         dessin.etablirDessin();
         dessin.paint(g);
      }
   ///////////////////////////////////////////////////////////////////////////////////
   ////// Methode ajoute pour la colaboration de cette classe avec "diva.*"  /////////
   
    /**
     * Cre un noeud contenant des donnes d'utilisateur.
     */
      public Noeud(Object userObject) {
         setSemanticObject(userObject);
      }
   }
