/*
 * $Id: GraphWriter.java,v 1.1.1.1 2000/02/29 14:14:43 tanguy Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
   package carto.parse;
   
   import diva.graph.model.CompositeNode;
   import diva.graph.model.Graph;
   import diva.graph.model.GraphModel;
   import diva.graph.model.Node;
   import diva.graph.model.Edge;

   import java.io.DataOutputStream;
   import java.io.FileOutputStream;
   import java.io.IOException;
   import java.util.HashMap;
   import java.util.Iterator;
   import java.util.Vector;

   import carto.graphe.*;

/**
 * Given a graph model, GraphWriter writes out the graph data
 * structure in XML format.
 *
 * @author Heloise Hse  (hwawen@eecs.berkeley.edu)
 * @author Claudia
 * @version $Revision: 1.1.1.1 $
 */
   public class GraphWriter {
   
    /**
     * Write the specified graph model to the output stream.
     */
      public void write(GraphModel model, DataOutputStream out)
      throws IOException {
         writeHeader(out);
         out.writeBytes("<Graphe>\n");
      
        //write Meta
         writeMeta(model.getGraph(), out); 
      
        //write Noeud
         writeNodes(model.getGraph(), out);
      
        //write Lien
         writeEdges(model.getGraph(), out);
      
        //write Chemin
         writeChemin(model.getGraph(), out);
      
         out.writeBytes("</Graphe>\n");
      }
   
      private void writeNodes(Graph graph, DataOutputStream out)
      throws IOException {
         for(Iterator ns = graph.nodes(); ns.hasNext();){
            Node n = (Node)ns.next();
            if(n instanceof Noeud){
               out.writeBytes("<Noeud>\n");
               out.writeBytes(((Noeud)n).toXML());
               out.writeBytes("</Noeud>\n");
            }
         }
      }
   
      private void writeMeta(Graph graph, DataOutputStream out)
      throws IOException {
         if ( graph instanceof Graphe) {
            out.writeBytes("<Meta>\n");
            out.writeBytes(((Graphe)graph).meta.toXML());
            out.writeBytes("</Meta>\n");
         }
      }
   
      private void writeEdges(Graph graph, DataOutputStream out)
      throws IOException {
         Vector vecteurLien = ((Graphe) graph).getVecteurLien();
         for (int i = 0; i<vecteurLien.size(); i++)
         {
            Lien lien = (Lien) vecteurLien.elementAt(i);
            out.writeBytes("<Lien>\n");
         	// pour le cas d'un lien bidirectionnel, on ecrit les deux liens 
         	// unidirectionnels contenus
            if (lien instanceof LienBiDir ) {
               out.writeBytes(((LienBiDir)lien).premier().toXML());
               out.writeBytes("</Lien>\n\t<Lien>\n");
               out.writeBytes(((LienBiDir)lien).deuxieme().toXML());
            }
            else out.writeBytes(((LienUniDir)lien).toXML());
            out.writeBytes("</Lien>\n");
         }
      
      }
   
      private void writeChemin(Graph graph, DataOutputStream out)
      throws IOException {
         Vector vecteurChemin = ((Graphe) graph).getVecteurChemin();
         for (int i = 0; i<vecteurChemin.size(); i++)
         {
            out.writeBytes("<Chemin>\n");
            out.writeBytes( ((Chemin)vecteurChemin.elementAt(i)).toXML() );
            out.writeBytes("</Chemin>\n");
         }      
      }
   
    /**
     * Write the header of the xml file.
     */
      private void writeHeader(DataOutputStream out) throws IOException {
         out.writeBytes("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>\n");
         out.writeBytes("<!DOCTYPE Graphe [\n");
         out.writeBytes("<!ELEMENT Graphe (Meta, Noeud+, Lien*, Chemin*)>\n\n");
         out.writeBytes("<!ELEMENT Meta (Titre, Createur, MotsCles, Date, Version, Niveau, Pere)>\n");
         out.writeBytes("<!ELEMENT Titre (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Createur (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT MotsCles (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Date (An, Mois, Jour)>\n");
         out.writeBytes("<!ELEMENT An (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Mois (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Jour (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Version (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Niveau (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Pere (#PCDATA)>\n\n");
         out.writeBytes("<!ELEMENT Noeud (TypeN, Ident, Nom, Symbole, Support, URL, Volume, Difficulte, CoordX, CoordY, Zoom)>\n");
         out.writeBytes("<!ELEMENT TypeN (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Ident (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Nom (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Symbole (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Support (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT URL (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Volume (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Difficulte (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT CoordX (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT CoordY (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Zoom (#PCDATA)>\n\n");
         out.writeBytes("<!ELEMENT Lien (noeudD, noeudA, TypeL, Diff)>\n\n");
         out.writeBytes("<!ELEMENT noeudD (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT noeudA (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT TypeL (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Diff (#PCDATA)>\n\n");
         out.writeBytes("<!ELEMENT Chemin (NomChemin, Cours, LienChemin+)>\n");
         out.writeBytes("<!ELEMENT NomChemin (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT LienChemin (NoeudD, NoeudA, Cours)>\n");
         out.writeBytes("<!ELEMENT NoeudD (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT NoeudA (#PCDATA)>\n");
         out.writeBytes("<!ELEMENT Cours (#PCDATA)> ]>\n");
         out.writeBytes("\n");
      }
   
   }
