// *********************************************
// Modificications pour mise en libre :
// Modif F le 13/06/2000 par Marc Dauplait :
// Quelques modifications d'interfaces pour compatibilite SAX2 et parser XERCES
// *********************************************

   package carto.parse;   

   import carto.graphe.Graphe;

   import org.apache.xerces.framework.XMLParser;		// Modif F
   import org.xml.sax.InputSource;						// Modif F
   import org.xml.sax.XMLReader;						// Modif F
   import org.xml.sax.helpers.XMLReaderFactory;			// Modif F
   import org.xml.sax.helpers.DefaultHandler;			// Modif F

   import java.io.*;

/**
 * La classe qui s'occuppe de la conversion d'un fichier XML.
 */
   public class GrapheParser
   {
      static final String parserClass = "org.apache.xerces.parsers.SAXParser"; // Modif F
      static final String fichier = "test.xml";
      private XMLReader parser;
      private DefaultHandler handler;
   
  /**
   * Cre un objet GrapheParser et l'initialise.
   */   
      public GrapheParser() throws Exception
      {  //cree le parseur en utilisant 'org.apache.xerces.parsers.SAXParser' // Modif F
         //specifie et 'set' le Handler
         parser = XMLReaderFactory.createXMLReader(parserClass);			// Modif F
         handler = new ParseHandler();										// Modif F
         parser.setContentHandler(handler);
      } 
   
   /** 
    *  La mthode parse un ficher .xml et cre une instance
    *  de la classe 'carto.graphe.Graphe' avec tous ses noeuds, liens et chemins.
    *  @return une instance de la classe 'Graphe'
    *  @param unFichier le nom/ le rpertoire d'un fichier
    */
      public Graphe parse(String unFichier) throws Exception
      {
         parser.parse(unFichier);
         Graphe graphe=ParseHandler.graphe;
         return graphe;
      }	
   
   /** 
    *  La mthode parse un 'InputStream' contenant du XML et cre une instance
    *  de la classe 'carto.graphe.Graphe' avec tous ses noeuds, liens et chemins.
    *  @return une instance de la classe 'Graphe'
    *  @param reader la rprsentation Reader d'un fichier .xml
    */
      public Graphe parse(Reader reader) throws Exception
      { //la classe 'com.ibm.xml.framework.XMLParser' sait parser des fichier 
        //et des instances de 'org.xml.sax.InputSource'.
        //Alors, on fait un 'InputSource' du 'Reader' avant le 'parse(...)'.
         InputSource source = new InputSource(reader);
         parser.parse(source);
         Graphe graphe=ParseHandler.graphe;
         return graphe;
      }		
   
   /** 
    * La mthode parse un 'InputStream' contenant du .xml et cre une instance
    * de la classe 'carto.graphe.Graphe' avec tous ses noeuds, liens et chemins.
    * @return une instance de la classe 'Graphe'
    * @param stream la rprsentation Stream d'un fichier .xml
    */
      public Graphe parse(InputStream stream) throws Exception
      { //la classe 'com.ibm.xml.framework.XMLParser' sait parser des fichier 
        //et des instances de 'org.xml.sax.InputSource'.
        //Alors, on fait un 'InputSource' du 'InputStream' avant le 'parse(...)'.
         InputSource source = new InputSource(stream);
         parser.parse(source);
         Graphe graphe=ParseHandler.graphe;
         return graphe;
      }	
   
      public static void main (String args[])
      {//test pour l'utilisation de la classe 'java.io.StringReader'
         try
         {
            GrapheParser gp = new GrapheParser();
            BufferedReader br = new BufferedReader(new InputStreamReader(
         		new FileInputStream(new File(fichier))));
            String s, s2 = new String ();
            while ((s = br.readLine())!= null)
               s2 += s + "\n";
            br.close();
         
           //reconvertir en 'StringReader' pour que le parseur sache le traiter
            StringReader sr = new StringReader(s2);
            Graphe graphe=gp.parse(sr);
            System.out.println(graphe.toString());
         }
            catch (Exception e){
               System.out.println(e.toString());
            }
      }
   }