/*

 * $Id: BasicManhattanRouter.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.canvas.connector;



import diva.canvas.Figure;

import diva.canvas.Site;

import diva.util.java2d.Polyline2D;

import java.awt.Shape;



/** A basic manhattan router.

 *

 * <p> FIXME: Currently this class doesn't actually do Manhattan

 * routing.

 *

 * @version $Revision: 1.1.1.1 $

 * @author  Michael Shilman (michaels@eecs.berkeley.edu)

 * @author  John Reekie (johnr@eecs.berkeley.edu)

 */

public class BasicManhattanRouter implements ManhattanRouter {

    



    /** Reroute the given Shape, given that the head site moved.

     */

    public void rerouteHead (Connector c, Shape s) {

        Polyline2D line = (Polyline2D) s;

        

        line.setX(1, c.getHeadSite().getX());

        line.setY(1, c.getHeadSite().getY());

    }



    /** Reroute the given Shape, given that the tail site moved.

     */

    public void rerouteTail (Connector c, Shape s) {

        Polyline2D line = (Polyline2D) s;

        line.setX(0, c.getHeadSite().getX());

        line.setY(0, c.getHeadSite().getY());

    }



    /** Reroute the given shape, given that both the head the tail

     * sites moved. The shape is modified by the router.

     */

    public void reroute (Connector c, Shape s) {

        rerouteHead(c, s);

        rerouteTail(c, s);

    }



    /** Route the given connector, returning a Polyline2D. This

     * method is the same as route(), except that the return

     * type is tighter.

     */

    public Shape route (Connector c) {

        return routeManhattan(c);

    }



    /** Route the given connector, returning a Polyline2D. This

     * method is the same as route(), except that the return

     * type is tighter.

     */

    public Polyline2D routeManhattan (Connector c) {

        return new Polyline2D.Double (

                c.getHeadSite().getX(),

                c.getHeadSite().getY(),

                c.getTailSite().getX(),

                c.getTailSite().getY());

    }

}



