/*

 * $Id: CenterTarget.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */

package diva.canvas.connector;



import diva.canvas.Figure;

import diva.canvas.Site;

import diva.util.Filter;



import java.util.HashMap;



/** An implementation of connector targets that finds center sites.

 *

 * @version $Revision: 1.1.1.1 $

 * @author John Reekie (johnr@eecs.berkeley.edu)

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 */

public class CenterTarget implements ConnectorTarget {



    /** The mapping from figures to sites. Ignore the

     * problem that none of the contents will ever get

     * garbage-collected.

     */

    private HashMap _siteMap = new HashMap();



    /** Accept a figure. A site will only be returned for figures that

     * are accepted by this method. By default, any figure will be accepted

     * except for Connectors.

     */

    public boolean accept (Figure f) {

        return !(f instanceof Connector);

    }



    /** Return a center site located on the figure, if the

     * figure passes the filter.

     */

    public Site getHeadSite (Figure f, double x, double y) {

        if (accept(f)) {

            if (_siteMap.containsKey(f)) {

                return (Site) _siteMap.get(f);

            } else {

                Site s = new CenterSite(f);

                _siteMap.put(f, s);

                return s;

            }

        }

        return null;

    }



    /** Return a center site located on the figure

     */

    public Site getTailSite (Figure f, double x, double y) {

        return getHeadSite(f, x, y);

    }



    /** Return the same site

     */

    public Site getHeadSite (Site s, double x, double y) {

        return s;

    }



    /** Return the same site

     */ 

    public Site getTailSite (Site s, double x, double y) {

        return s;

    }

}

