/*

 * $Id: Connector.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */



package diva.canvas.connector;



import diva.canvas.Figure;

import diva.canvas.Site;

import java.awt.geom.Point2D;



/** A Connector is a figure that draws itself between two

 * sites on other figures. To do so, it requires two references

 * to sites on other figures.

 *

 * @version $Revision: 1.1.1.1 $

 * @author  Michael Shilman (michaels@eecs.berkeley.edu)

 * @author  John Reekie (johnr@eecs.berkeley.edu)

 */

public interface Connector extends Figure {



    /** Get the site that marks the "head" of the connector.

     */

    public Site getHeadSite ();



    /** Get the label of this connector. The returned value

     * may be null.

     */

    //// public String getLabel ();



    /** Get the site that marks the "tail" of the connector.

     */

    public Site getTailSite ();



    /** Inform the connector that the head site has moved.

     * The connector is expected to reroute itself using

     * an efficient means as is available.

     */

    public void headMoved ();



    /** Tell the connector to re-route itself. In general,

     * implementations of this method should be more efficient

     * than route().

     */

    public void reroute ();



    /** Tell the connector to route itself completely,

     * using all available information.

     */

    public void route ();



    /** Set the site that marks the "head" of the connector.

     */

    public void setHeadSite (Site s);



    /** Set the site that marks the "tail" of the connector.

     */

    public void setTailSite (Site s);



    /** Set the label of this connector. All connectors are

     * assumed to be able to display at least one label

     * in a useful way.

     */

    //// public void setLabel (String s);



    /** Inform the connector that the tail site has moved.

     * The connector is expected to reroute itself using

     * an efficient means as is available.

     */

    public void tailMoved ();

}

