/*

 * $Id: PerimeterTarget.java,v 1.1.1.1 2000/05/15 22:11:03 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 *

 */

package diva.canvas.connector;



import diva.canvas.Figure;

import diva.canvas.Site;

import diva.util.Filter;



import java.util.HashMap;



/** A connector target that returns sites on the perimeter of a figure.

 *

 * @version $Revision: 1.1.1.1 $

 * @author John Reekie (johnr@eecs.berkeley.edu)

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 */

public class PerimeterTarget implements ConnectorTarget {



    /** Accept a figure. A site will only be returned for figures that

     * are accepted by this method. By default, any figure will be accepted

     * except for Connectors.

     */

    public boolean accept (Figure f) {

        return !(f instanceof Connector);

    }



    /** Return the nearest site on the figure if the figure

     * is not a connector

     */

    public Site getHeadSite (Figure f, double x, double y) {

	if (accept(f)) {

	    // FIXME: Need to generate unique ID per figure

	    // FIXME: Need to actually return a useful site!

	    return new PerimeterSite(f, 0, 1.0);

	} else {

	    return null;

	}

    }



    /** Return the nearest site on the figure

         */

    public Site getTailSite (Figure f, double x, double y) {

	return getHeadSite(f, x, y);

    }



    /** Return the nearest site on the site's figure

         */

    public Site getHeadSite (Site s, double x, double y) {

	//  return getHeadSite(s.getFigure(), x, y);

	return s;

    }



    /** Return the nearest site on the site's figure

         */

    public Site getTailSite (Site s, double x, double y) {

	// return getHeadSite(s, x, y);

	return s;

    }

}

