/*
 * $Id: AbstractGraphController.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
   package diva.graph;

   import diva.graph.model.GraphModel;
   import diva.graph.model.Graph;
   import diva.graph.model.GraphEvent;
   import diva.graph.model.GraphUtilities;
   import diva.graph.model.Node;
   import diva.graph.model.Edge;

   import diva.canvas.Figure;
   import diva.canvas.FigureLayer;
   import diva.canvas.GraphicsPane;
   import diva.canvas.Site;

   import diva.canvas.connector.AutonomousSite;
   import diva.canvas.connector.CenterSite;
   import diva.canvas.connector.Connector;
   import diva.canvas.connector.ConnectorTarget;
   import diva.canvas.connector.CenterTarget;

   import diva.canvas.event.LayerAdapter;
   import diva.canvas.event.LayerEvent;
   import diva.canvas.event.MouseFilter;

   import diva.canvas.selection.SelectionModel;
   import diva.canvas.selection.BasicSelectionModel;

   import java.util.Iterator;
   import java.awt.geom.Rectangle2D;
   
   import carto.editor.*;

/**
 * An implementation of GraphController that provides some useful
 * default method implementations.
 *
 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)
 * @version	$Revision: 1.1.1.1 $
 * @rating      Red
 */
   public abstract class AbstractGraphController implements GraphController {
   
    /** The connector target
     */
      private ConnectorTarget _connectorTarget = new CenterTarget();
   
    /** The interactor for edges.
     */
      private EdgeInteractor _edgeInteractor;
   
    /** The interactor for nodes.
     */
      private NodeInteractor _nodeInteractor;
   
    /** The graph model
     */
      protected GraphModel _model;
   
    /** The default selection model
     */
      private SelectionModel _selectionModel = new BasicSelectionModel();
   
    /** The graph view
     */
      private GraphView _view;   
      
     //pour pouvoir donner la reference du frame aux fenetres de dialogue
     //@author Claudia
      protected EditeurFrame frame;
   
    /**
     * Test if both ends of the edge are visible in the view.
     * The test returns true if both ends either: i) are connected
     * to a visible node, or ii) are connected to an autonomous
     * site and the edge figure is visible,
     */
    /*private*/ 
      boolean areEndsVisible(Edge e) {
         Node hn = e.getHead();
         Node tn = e.getTail();
      
         Connector c = _view.getEdgeFigure(e);
      
        // debug("hn = " + hn);
        // debug("hfig = " + _view.getNodeFigure(hn));
        // debug("tn = " + tn);
        // debug("tfig = " + _view.getNodeFigure(tn));
        // debug("Edge = " + e + ", figure = " + c);
        // if (c != null) {
        //    debug("Head site = " + c.getHeadSite() +
        //             ", tail site = " + c.getTailSite());
        // }
      
         boolean hasHeadNode = (hn != null) && (_view.getNodeFigure(hn) != null);
         boolean hasAutoHead = (c != null) && (c.getHeadSite() instanceof AutonomousSite);
         boolean hasTailNode = (tn != null) && (_view.getNodeFigure(tn) != null);
         boolean hasAutoTail = (c != null) && (c.getTailSite() instanceof AutonomousSite);
      
      //          debug("areEndsVisible: " + e + "[ "
      //                  + hasHeadNode + ", " + hasAutoHead + ", "
      //                  + hasTailNode + ", " + hasAutoTail + "]");
      
         return (hasHeadNode || hasAutoHead) && (hasTailNode || hasAutoTail);
      }
   
    /**
     * Create an edge figure for this edge, add it to the view, and route it.
     */
    /*private*/ 
      void createEdgeFigure(Edge e) {
        // Get a tail site
         Figure nf =  _view.getNodeFigure(e.getTail());
         Rectangle2D bounds = nf.getBounds();
         Site tailSite = getConnectorTarget().getTailSite(nf,
                                             bounds.getCenterX(), bounds.getCenterY());
      
        // Get a head site
         nf =  _view.getNodeFigure(e.getHead());
         bounds = nf.getBounds();
         Site headSite = getConnectorTarget().getHeadSite(nf,
                                             bounds.getCenterX(), bounds.getCenterY());
      
        // Create the figure
         Connector ef = _view.getEdgeRenderer().render(e, tailSite, headSite);
         ef.setInteractor(getEdgeInteractor());
      
        // Add it to the view
         _view.addEdgeFigureToLayer(ef);
         _view.addEdgeMapping(e, ef);
      
        // Re-route it
         ef.route();
      }
   
    /* Print debug message.
     */
      private void debug(String s) {
         System.err.println(s);
      }
   
    /**
     * Respond to a change in the head of an edge. If the
     * edge was not visible before (there was no connector
     * displaying it), but it should be now, create the connector.
     * If it was not visible and still shouldn't be, do nothing.
     * If it was visible but now should <i>not</i> be, remove 
     * the figure. Otherwise, check whether the head node is
     * not null, and if not, connect the edge to the current
     * head node.
     */
      public void edgeHeadChanged(GraphEvent evt) {
         Edge e = (Edge)evt.getTarget();
         boolean wasVisible = _view.isEdgeVisible(e);
         boolean endsVisible = areEndsVisible(e);
        // debug("Edge " + e + ": wasVisible = " + wasVisible + ", endsVisible = " + endsVisible);
      
         if (!wasVisible && !endsVisible) {
            // Do nothing if it wasn't visible and still isn't
         }
         else if (wasVisible && !endsVisible) {
            // Can't see it any more, so remove it
            removeEdgeFigure(e);
         }
         else if (!wasVisible && endsVisible) {
            // It became visible, so create the edge figure
            createEdgeFigure(e);
         }
         else {
            // Was and still is visible, so assume the head connection changed
            Connector ef = _view.getEdgeFigure(e);
            Site hs = ef.getHeadSite();
         
            // If it's still attached, get a site and attach to it
            if(e.getHead() != null) {
               Figure nf = _view.getNodeFigure(e.getHead());
               Site s = getConnectorTarget().getHeadSite(nf, hs.getX(), hs.getY());
               ef.setHeadSite(s);
            }
         }
        // Notify the layout
         _view.getIncrLayout().edgeHeadChanged(evt);
      }
   
    /**
     * Respond to a change in the tail of an edge. If the
     * edge was not visible before (there was no connector
     * displaying it), but it should be now, create the connector.
     * If it was not visible and still shouldn't be, do nothing.
     * If it was visible but now should <i>not</i> be, remove 
     * the figure. Otherwise, check whether the head node is
     * not null, and if not, connect the edge to the current
     * tail node.
     */
      public void edgeTailChanged(GraphEvent evt) {
         Edge e = (Edge)evt.getTarget();
         boolean wasVisible = _view.isEdgeVisible(e);
         boolean endsVisible = areEndsVisible(e);
        // debug("Edge " + e + ": wasVisible = " + wasVisible + ", endsVisible = " + endsVisible);
      
         if (!wasVisible && !endsVisible) {
            // Do nothing if it wasn't visible and still isn't
         }
         else if (wasVisible && !endsVisible) {
            // Can't see it any more, so remove it
            removeEdgeFigure(e);
         }
         else if (!wasVisible && endsVisible) {
            // It became visible, so create the edge figure
            createEdgeFigure(e);
         }
         else {
            // Was and still is visible, so assume the tail connection changed
            Connector ef = _view.getEdgeFigure(e);
            Site ts = ef.getTailSite();
         
            // If it's still attached, get a site and attach to it
            if(e.getTail() != null) {
               Figure nf = _view.getNodeFigure(e.getTail());
               Site s = getConnectorTarget().getTailSite(nf, ts.getX(), ts.getY());
               ef.setTailSite(s);
            }
         }
        // Notify the layout
         _view.getIncrLayout().edgeTailChanged(evt);
      }
   
    /**
     * Get the target used to find sites on nodes to connect to.
     */
      public ConnectorTarget getConnectorTarget () {
         return _connectorTarget;
      }
   
    /**
     * Get the interactor given to edge figures.
     */
      public EdgeInteractor getEdgeInteractor () {
         return _edgeInteractor;
      }
   
    /**
     * Get the graph model.
     */
      public final GraphModel getGraphModel () {
         return _model;
      }
   
    /**
     * Get the graph view.
     */
      public final GraphView getGraphView () {
         return _view;
      }
   
    /**
     * Get the interactor given to node figures.
     */
      public NodeInteractor getNodeInteractor () {
         return _nodeInteractor;
      }
   
    /**
     * Get the default selection model
     */
      public SelectionModel getSelectionModel () {
         return _selectionModel;
      }
   
    /**
     * Initialize all interaction on the graph pane. By the time
     * this method is called, all relevant references to views,
     * panes, and interactors must already have been set up.
     */
      public abstract void initializeInteraction ();
   
      public void edgeAdded(GraphEvent evt){
      //la methode 'addNode' ne sert que a ajouter le noeud au 'view'. 
      // Ajouter les liens au 'view' est (a mon avis) deja fait par 'addNode'.
      }
   
    /**
     * Add a node to the view. If the node is not already being displayed:
     * <ol>
     * <li>Render the node.
     * <li>Update all the edges that are attached to it, if they
     *     were previously "in limbo".
     * <li>Give the event to the incremental layout to handle.
     * </ol>
     *
     * If the node is reparented:
     * <ol>
     * <li>Reparent the node's figure.
     * <li>Give the event to the incremental layout to handle.
     * </ol>
     */
      public void nodeAdded(GraphEvent evt) {
         Node n = (Node)evt.getTarget();
         boolean doLayout = false;
        // debug("Trying to add: " + n);
      
        // Create the corresponding figure if it doesn't exist yet
         Figure nf = _view.getNodeFigure(n);
         if(nf == null) {
            nf = _view.getNodeRenderer().render(n);
            _view.addNodeMapping(n, nf);
            doLayout = true;
         }
        // Set the node interactor always
         nf.setInteractor(getNodeInteractor());
      
        // If the node figure is not in the display tree, either it was
        // newly created or it was "set" by the user but is not yet displayed
        // on the layer. So display it.
         if(nf.getParent() == null) {
            if(n.getParent() == _model.getGraph()) {
               _view.addNodeFigureToLayer(nf);
            }
            else {
               _view.addNodeFigureToParent(nf);
            }
            // Display attached edges
            for(Iterator i = n.outEdges(); i.hasNext();) {
               Edge e = (Edge)i.next();
               if(areEndsVisible(e)) {
                  createEdgeFigure(e);
               }
            }
            // Refresh the layout
            if(doLayout) {
               _view.getIncrLayout().nodeAdded(evt);
            }
         }
         else {
            //the node was already being displayed, so we only
            //relayout if the parent has changed and was non-null
            //before.
            //
            //FIXME: also must reparent!
            Graph prevParent = (Graph)evt.getOldValue();
            if((prevParent != n.getParent()) && (prevParent != null)) {
               _view.getIncrLayout().nodeAdded(evt);
            }
         }
      }
   
    /**
     * Remove a node from the view.
     * If we have a view of this node, remove it.  Hide
     * all the edges that were connected to this node.
     */
      public void nodeRemoved(GraphEvent evt) {
         Node n = (Node)evt.getTarget();
      
         Figure nf = _view.getNodeFigure(n);
         if(nf != null) {
            _view.removeNodeMapping(n, nf);
            _view.getGraphicsPane().getForegroundLayer().remove(nf);
         }
      
         for(Iterator i = n.inEdges(); i.hasNext(); ) {
            Edge e = (Edge)i.next();
            removeEdgeFigure(e);
         }
         for(Iterator i = n.outEdges(); i.hasNext(); ) {
            Edge e = (Edge)i.next();
            removeEdgeFigure(e);
         }
      
         if(nf != null) {
            _view.getIncrLayout().nodeRemoved(evt);
         }
      }
   
    /** Rebuild the complete view from the model.
     */
    /*private*/ 
      void refreshRecursive(Graph g, boolean isRoot) {
         for(Iterator i = g.nodes(); i.hasNext(); ) {
            Node n = (Node)i.next();
            Figure nf = _view.getNodeRenderer().render(n);
            nf.setInteractor(getNodeInteractor());
            _view.addNodeMapping(n, nf);
            if(isRoot) {
               _view.addNodeFigureToLayer(nf);
            }
            else {
               _view.addNodeFigureToParent(nf);
            }
            if(n instanceof Graph) {
               Graph sub = (Graph)n;
               refreshRecursive(sub, false);
            }
         }
         for(Iterator i = GraphUtilities.localEdges(g); i.hasNext(); ) {
            Edge e = (Edge)i.next();
            createEdgeFigure(e);
         }
      }
   
    /**
     * Set the target used to find sites on nodes to connect to.
     */
      public void setConnectorTarget (ConnectorTarget t) {
         _connectorTarget = t;
      }
   
    /**
     * Set the interactor given to edge figures.
     */
      protected void setEdgeInteractor (EdgeInteractor interactor) {
         _edgeInteractor = interactor;
      }
   
    /**
     * Set the view that this controller operates on. This method
     * sets the view, and calls initializeInteraction().
     */
      public void setGraphView (GraphView view) {
         _view = view;
         initializeInteraction();
      }
   
    /**
     * Set the graph model that is being viewed. Unsubscribe from
     * the previous one if there is one. Then subscribe to the new one.
     */
      public void setGraphModel(GraphModel m) {
         if(_model != null) {
            _model.removeGraphListener(this);
         }
         _model = m;
      
         if(_model != null) {
            _model.addGraphListener(this);
         
            //force an update
            GraphEvent update = new GraphEvent(
                              GraphEvent.STRUCTURE_CHANGED, this, _model.getGraph());
            structureChanged(update);
         }
      }
   
    /**
     * Set the interactor given to node figures. This interactor will be
     * given to all newly created figures.
     */
      protected void setNodeInteractor (NodeInteractor interactor) {
         _nodeInteractor = interactor;
      }
   
    /**
     * Set the default selection model. The caller is expected to ensure
     * that the old model is empty before calling this.
     */
      public void setSelectionModel (SelectionModel m){
         _selectionModel = m;
      }
   
    /**
     * Remove all the node and edge figures from this pane.
     * Go through the graph and add the nodes and edges.
     */
      public void structureChanged(GraphEvent evt) {
        // Remove all figures
         _view.clear();
      
        // Rebuild all figures
         refreshRecursive((Graph)evt.getTarget(), true);
      
        // Force re-layout
         _view.getIncrLayout().structureChanged(evt);
      
        // Repaint
         _view.getGraphicsPane().repaint();
      }
   
    /**
     * Remove an edge figure from the display or do
     * nothing if it wasn't there in the first place.
     */
      protected void removeEdgeFigure(Edge e) {
         Connector ef = _view.getEdgeFigure(e);
         _view.removeEdgeMapping(e, ef);
         if(ef != null) {
            _view.getGraphicsPane().getForegroundLayer().remove(ef);
         }
      }
   }
