/*

 * $Id: CompositeNodeFigure.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph;

import diva.graph.layout.LayoutTarget;

import diva.graph.model.Node;

import diva.graph.model.Graph;

import diva.graph.model.Edge;

import diva.graph.model.CompositeNode;

import diva.canvas.DamageRegion;

import diva.canvas.FigureWrapper;

import diva.canvas.Figure;

import diva.canvas.CompositeFigure;

import diva.canvas.toolbox.BasicFigure;

import diva.canvas.Site;

import diva.canvas.connector.CenterSite;

import java.awt.Shape;

import java.awt.geom.AffineTransform;

import java.awt.geom.Point2D;

import java.awt.geom.Rectangle2D;

import java.util.Iterator;



/**

 * A basic implementation of NodeFigure for composite

 * nodes.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class CompositeNodeFigure extends CompositeFigure {

    Figure _background;

    

    /**

     * Construct a node figure whose "model" is the given node using

     * the default representation of a 50x50 rectangle.

     */

    public CompositeNodeFigure(Node n) {

        this(n, new Rectangle2D.Double(0.0,0.0,50.0,50.0));

    }



    /**

     * Construct a node figure whose "model" is the given node, using

     * the given shape as its visual representation.

     */

    public CompositeNodeFigure(Node n, Shape s) {

        this(n, new BasicFigure(s));

    }



    /**

     * Construct a node figure whose "model" is the given node, using

     * the given figure as the node's representation.

     */

    public CompositeNodeFigure(Node n, Figure f) {

        super();

        setBackgroundFigure(f);

        setUserObject(n);

    }



    /**

     * Debugging output.

     */

    private void debug(String s) {

        System.out.println(s);

    }



    /**

     * Return the node of which this figure is a visual

     * representation.

     */

    public Node getNode() {

        return (Node)getUserObject();

    }



    /**

     * Get repaint notification. Override the superclass method to

     * reroute edges. (This is inefficient, but it will do for now).

     */

    public void repaint(DamageRegion d) {

        if (getLayer() != null && getLayer().getCanvasPane() != null ) {

            super.repaint(d);

        }

    }



    public Figure getBackgroundFigure() {

        return _background;

    }



    public void setBackgroundFigure(Figure f) {

        _background = f;

        add(f);

    }



    /**

     * Transform the node with the supplied transform.  Any edges

     * connected to the node are reshaped accordingly.

     */

    public void transform(AffineTransform at) {

        super.transform(at);

    }



    /**

     * Translate the node the given distance. Any edges connected to

     * the node are reshaped accordingly.

     */

    public void translate(double dx, double dy) {

        super.translate(dx, dy);

    }

    

    /**

     * Return a site at the center of this figure.

     * This is a temporary method until we figure out

     * how this should work generally.

     */

    public Site getCenterSite() {

         return new CenterSite(this);

    }

}

