/*

 * $Id: GraphPane.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph;



import diva.graph.model.GraphModel;

import diva.graph.layout.LayoutTarget;

import diva.canvas.GraphicsPane;



/**

 * The display part of the JGraph user-level widget.

 *

 * @see GraphModel

 * @see JGraph

 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)

 * @version	$Revision: 1.1.1.1 $

 * @rating Red

 */

public class GraphPane extends GraphicsPane {



    /** The view

     */

    private GraphView _view;



    /** The model

     */

    private GraphModel _model;



    /** The controller

     */

    private GraphController _controller;



    /** Create a new graph pane with a default view, model,

     * and controller.

     */

    public GraphPane () {

        this(   new GraphModel(),

                new GraphView(),

                new BasicGraphController());

    }



    /** Create a new graph pane with the given model, and a default

     * view and controller.

     */

    public GraphPane (GraphModel model) {

        this(   model,

                new GraphView(),

                new BasicGraphController());

    }



    /** Create a new graph pane with the given model and controller.

     */

    public GraphPane (GraphModel model, GraphController controller) {

        this(   model,

                new GraphView(),

                controller);

    }



    /** Create a new graph pane with the given model,

     * view and controller.

     */

    public GraphPane (GraphModel model,

            GraphView view,

            GraphController controller) {



        // Set up the view

        _view = view;

        _view.setGraphPane(this);

        _view.setGraphController(controller);



        // Set up the controller

        _controller = controller;

        _controller.setGraphView(view);



        //Set up the model

        _model = model;

        setGraphModel(model);

    }



    /** Get the graph controller

     */

    public GraphController getGraphController () {

        return _controller;

    }



    /** Get the graph model

     */

    public GraphModel getGraphModel () {

        return _model;

    }



    /** Get the graph view

     */

    public GraphView getGraphView () {

        return _view;

    }



    /** Set the graph model. Any existing data in the view will be

     * removed, and the new data loaded into the model.

     */

    public void setGraphModel (GraphModel m) {

        _model = m;



        // Inform the view

        _view.setGraphModel(_model);



        // Inform the controller. It will subscribe to the model

        // and cause the view to be updated.

        _controller.setGraphModel(_model);

    }



    /**

     * Implements LayoutPane; returns the GraphView.

     */

    // FIXME: No longer needed

    //public LayoutTarget getLayoutTarget() {

    //    return getGraphView();

    //}

}

