/*
 * $Id: NodeDragInteractor.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $
 *
 * Copyright (c) 1998 The Regents of the University of California.
 * All rights reserved.  See the file COPYRIGHT for details.
 */
package diva.graph;

import diva.graph.model.Edge;
import diva.graph.model.Node;
import diva.graph.model.CompositeNode;

import diva.canvas.Figure;
import diva.canvas.connector.Connector;
import diva.canvas.event.LayerEvent;
import diva.canvas.interactor.DragInteractor;
import diva.canvas.selection.SelectionModel;

import carto.graphe.Noeud;

import java.util.Iterator;

/**
 * An interaction role that drags nodes.
 *
 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)
 * @author 	John Reekie (johnr@eecs.berkeley.edu)
 * @version	$Revision: 1.1.1.1 $
 * @rating Red
 */
public class NodeDragInteractor extends DragInteractor {

    /** The controller that this interactor is a part of
     */
    private GraphController _controller;

    /** Create a new NodeDragInteractor and give it a pointer
     * to its controller to it can find other useful objects
     */
    public NodeDragInteractor (GraphController c) {
        _controller = c;
    }

    /** Drag all selected nodes and move any attached edges
     */
    public void translate (LayerEvent e, double x, double y) {
        Iterator i = targets();
        while (i.hasNext()) {
            Figure t = (Figure) i.next();

	    // Translate anything that is not an edge
	    // FIXME: we want to drag edges that are not connected
	    // at either end
	    if ( !(t.getUserObject() instanceof Edge) ) {
		t.translate(x,y);
	    }
	    // Route the attached edges
            if (t.getUserObject() instanceof Node) {
                Node node = (Node) t.getUserObject();
                routeNode(node);
                
                //memoriser les nouvelles coordonnees du noeud
                // @author Claudia
                if ( node instanceof Noeud ) {
                	((Noeud) node).setCoordX( (int)x + ((Noeud) node).getCoordX() );
                	((Noeud) node).setCoordY( (int)y + ((Noeud) node).getCoordY() );
                }
            }
        }
    }

    private void routeNode(Node n) {
        // Translate the attached edges. This is inefficient
        // and needs to be improved!
        GraphView v = _controller.getGraphView();
        
        // Reroute attached edges. We have to check for null
        // figures, since if an edge is connected at only one end
        // in one view, other views will have no edge figure
        // for that edge.
        for (Iterator ins = n.inEdges(); ins.hasNext(); ) {
            Connector c = v.getEdgeFigure((Edge) ins.next());
            if (c != null) {
                c.reroute();
            }
        }
        for (Iterator outs = n.outEdges(); outs.hasNext(); ) {
            Connector c = v.getEdgeFigure((Edge) outs.next());
            if (c != null) {
                c.reroute();
            }
        }

        if(n instanceof CompositeNode) {
            routeRecursive((CompositeNode)n);
        }
    }

    private void routeRecursive(CompositeNode cn) {
        for(Iterator i = cn.nodes(); i.hasNext();) {
            routeNode((Node)i.next());
        }
    }
}
