/*

 * $Id: NodeInteractor.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph;



import diva.canvas.Figure;

import diva.canvas.event.LayerEvent;

import diva.canvas.interactor.DragInteractor;

import diva.canvas.selection.SelectionModel;

import diva.canvas.selection.SelectionInteractor;



import java.util.Iterator;



/**

 * An interaction role for nodes.

 *

 * @author 	Michael Shilman (michaels@eecs.berkeley.edu)

 * @author 	John Reekie (johnr@eecs.berkeley.edu)

 * @version	$Revision: 1.1.1.1 $

 * @rating Red

 */

public class NodeInteractor extends SelectionInteractor {



    /** The controller that this interactor is a part of

     */

    private GraphController _controller;



    /** The interactor that drags nodes

     */

    private DragInteractor _nodeDragInteractor;



    /** Create a new node interactor that belongs to the given

     * controller.

     */

    public NodeInteractor (GraphController c) {

        super();

        _controller = c;

        setDragInteractor(new NodeDragInteractor(c));

    }



    /** Create a new node interactor that belongs to the given

     * controller and that uses the given selection model

     */

    public NodeInteractor (GraphController c, SelectionModel sm) {

        this(c);

        super.setSelectionModel(sm);

    }



    /** Get the interactor that drags nodes

     */

    public DragInteractor getDragInteractor () {

        return _nodeDragInteractor;

    }



    /** Set the interactor that drags nodes

     */

    public void setDragInteractor (DragInteractor i) {

        if (_nodeDragInteractor != null) {

            removeInteractor(_nodeDragInteractor);

        }

        addInteractor(i);

        _nodeDragInteractor = i;

    }

}

