/*

 * $Id: TypedNodeRenderer.java,v 1.1.1.1 2000/05/15 22:11:06 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph;

import diva.graph.model.Node;

import diva.canvas.Figure;

import java.util.Hashtable;



/**

 * A NodeRenderer implementation which allows a user

 * to add different specialized node renderers which

 * are called selectively, based on the type of the

 * user object that is contained in the node that is

 * being rendered.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class TypedNodeRenderer implements NodeRenderer {

    /**

     * The default renderer.

     */

    private NodeRenderer _defaultRenderer = null;



    /**

     * The typed renderers.

     */

    private Hashtable _typedRenderers = null;



    /**

     * A typed node renderer with a DefaultNodeRenderer

     * as its default.

     */

    //    public TypedNodeRenderer() {

    //    this(new BasicNodeRenderer());

    //}



    /**

     * A typed node renderer with the given node renderer

     * as its default.

     */

    public TypedNodeRenderer(NodeRenderer defaultRenderer) {

        _defaultRenderer = defaultRenderer;

        _typedRenderers = new Hashtable();

    }



    /**

     * Add a renderer which is invoked when the rendered

     * node's user object is an instance of the given class.

     */

    public void addTypedRenderer(Class c, NodeRenderer r) {

        _typedRenderers.put(c, r);

    }



    /**

     * Remove a typed renderer.

     */

    public void removeTypedRenderer(Class c) {

        _typedRenderers.remove(c);

    }



    /**

     * Return the rendered visual representation of this node.

     */

    public Figure render(Node n) {

        Object o = n.getSemanticObject();

        if(o != null) {

            NodeRenderer r = (NodeRenderer)_typedRenderers.get(o.getClass());

            if(r != null) {

                return r.render(n);

            }

        }

        return _defaultRenderer.render(n);

    }

}

