 /*

 * $Id: IncrLayoutAdapter.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;

import diva.graph.model.*;

import java.awt.geom.*;



/**

 * An adapter to make global layouts incremental.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class IncrLayoutAdapter extends GraphAdapter implements IncrementalLayout {

    private LayoutTarget _target;

    private GlobalLayout _layout;



    public IncrLayoutAdapter(GlobalLayout l, LayoutTarget target) {

        _layout = l;

        _target = target;

    }



    public void nodeAdded(GraphEvent evt) {

        Node n = (Node)evt.getTarget();

        layout(n.getParent());

    }



    public void nodeRemoved(GraphEvent evt) {

        Node n = (Node)evt.getTarget();

        Graph g = (Graph)(evt.getOldValue());

        layout(g);

    }



    public void edgeHeadChanged(GraphEvent evt) {

        Edge e = (Edge)(evt.getTarget());

        layout(e.getHead().getParent());

    }



    public void edgeTailChanged(GraphEvent evt) {

        Edge e = (Edge)(evt.getTarget());

        layout(e.getTail().getParent());

    }



    public void structureChanged(GraphEvent evt) {

        layout((Graph)evt.getTarget());

    }



    private void layout(Graph g) {

	try {

            _layout.layout(_target, g);

	}

	catch(Exception e) {

	    System.err.println("Layout Failed: " + e.getMessage());

	}

    }

}

