/*

 * $Id: LayoutTarget.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;



import diva.graph.model.Edge;

import diva.graph.model.Graph;

import diva.graph.model.GraphModel;

import diva.graph.model.Node;

import java.awt.geom.Rectangle2D;

import java.util.Iterator;



/**

 * The basic set of information necessary to layout a graph: a mapping

 * the graph data structure to aspects of its visual representation,

 * a viewport to layout in, and some manipulation routines including

 * pick, place, and route. 

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public interface LayoutTarget {

    /**

     * Return the bounds of the figure associated with the given node

     * in the target's view.

     */

    public Rectangle2D getBounds(Node n);



    /**

     * Return the graph model that is being displayed by the view.

     */

    public GraphModel getGraphModel();



    /**

     * Return the viewport of the given graph as a rectangle

     * in logical coordinates.

     */

    public Rectangle2D getViewport(Graph g);



    /**

     * Return whether or not the given node is actually

     * visible in the view.

     */

    public boolean isNodeVisible(Node n);



    /**

     * Return whether or not the given edge is actually

     * visible in the view.

     */

    public boolean isEdgeVisible(Edge e);



    /**

     * Return an iterator over edges in the view which intersect

     * the given rectangle.

     */

    public Iterator intersectingNodes(Rectangle2D r);



    /**

     * Return an iterator over nodes in the view which intersect

     * the given rectangle.

     */

    public Iterator intersectingEdges(Rectangle2D r);



    /**

     * Route absolutely the figure associated with the given edge in

     * the target's view.

     */

    public void route(Edge e);



    /**

     * Reroute the figure associated with the given edge in the

     * target's view given an incremental change in one of its

     * endpoints.

     */

    //    public void reroute(Edge e);



    /**

     * Translate the figure associated with the given node in the

     * target's view by the given delta.

     */

    public void translate(Node n, double dx, double dy);

}

