/*

 * $Id: ProxyIncrLayout.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;

import diva.graph.model.GraphListener;

import diva.graph.model.GraphEvent;





/**

 * A proxy layout which forwards all events to a given incremental

 * layout "target".  This class implements the proxy design pattern,

 * hence its name.  It is useful for building layouts that know how

 * to do a certain specific task, but want to delegate work to a more

 * general layout for the cases that it dosn't know how to deal with.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class ProxyIncrLayout implements IncrementalLayout {

    /**

     * The target incremental layout that this delegates to.

     */

    private IncrementalLayout _target;



    /**

     * Construct the proxy with the given target.

     */

    public ProxyIncrLayout(IncrementalLayout target) {

        _target = target;

    }



    /**

     * Delegate the event to the target.

     */

    public void nodeAdded(GraphEvent evt) {

        _target.nodeAdded(evt);

    }



    /**

     * Delegate the event to the target.

     */

    public void nodeRemoved(GraphEvent evt) {

        _target.nodeRemoved(evt);

    }



    /**

     * Delegate the event to the target.

     */

    public void edgeHeadChanged(GraphEvent evt) {

        _target.edgeHeadChanged(evt);

    }



    /**

     * Delegate the event to the target.

     */

    public void edgeTailChanged(GraphEvent evt) {

        _target.edgeTailChanged(evt);

    }



    /**

     * Delegate the event to the target.

     */

    public void edgeAdded(GraphEvent evt) {

        _target.edgeAdded(evt);

    }



    /**

     * Delegate the event to the target.

     */

    public void structureChanged(GraphEvent evt) {

        _target.structureChanged(evt);

    }

}

