/*

 * $Id: RandomIncrLayout.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;

import diva.graph.model.*;
import carto.graphe.Noeud;

import java.awt.geom.*;

import java.util.Iterator;



/**

 * Randomly place newly-added nodes within the

 * target viewport.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class RandomIncrLayout extends GraphAdapter implements IncrementalLayout {

    private RandomLayout _global = new RandomLayout();

    private LayoutTarget _target;

    private static final int NUM_ITER = 10;



    public RandomIncrLayout(LayoutTarget target) {

        _target = target;

    }



    private void debug(String s) {

        System.out.println("RandomIncrLayout: " + s);

    }

    

    public void nodeAdded(GraphEvent evt) {
        Node n = (Node)evt.getTarget();

        if(_target.isNodeVisible(n)) {

            double x;
            double y;
            Rectangle2D vp = _target.getViewport(n.getParent());

            Rectangle2D bounds = _target.getBounds(n);

            for(int i = 0; i < NUM_ITER; i++) {

            	if (n instanceof Noeud ){
            	// @author Claudia
            		x = ((Noeud)n).getCoordX();
            		y = ((Noeud)n).getCoordY();
            		}
            	else {
                	x = vp.getX() + Math.abs(Math.random())*vp.getWidth();

                	y = vp.getY() + Math.abs(Math.random())*vp.getHeight();
                	}



                LayoutUtilities.place(_target, n, x, y);

                bounds = _target.getBounds(n);

                boolean overlap = false;

                Iterator j = _target.intersectingNodes(bounds);

                while(j.hasNext()) {

                    Node n2 = (Node)j.next();

                    if(n != n2) { overlap = false; }

                }

                if(!overlap) {

                    break;

                }

            }

        }

    }



    public void edgeHeadChanged(GraphEvent evt) {

        Edge e = (Edge)evt.getTarget();

        if(_target.isEdgeVisible(e)) {

            _target.route(e);

        }

    }



    public void edgeTailChanged(GraphEvent evt) {

        Edge e = (Edge)evt.getTarget();

        if(_target.isEdgeVisible(e)) {

            _target.route(e);

        }

    }



    public void structureChanged(GraphEvent evt) {

        Graph g = (Graph)(evt.getTarget());

        _global.layout(_target, g);

    }

}

