/*

 * $Id: RandomLayout.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.layout;

import diva.graph.model.*;
import carto.graphe.Noeud;

import java.util.Iterator;

import java.awt.geom.Rectangle2D;



/**

 * A static random layout engine.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class RandomLayout implements GlobalLayout {

    private static final int NUM_ITER = 10;



    /**

     * Layout the graph model and viewport

     * specified by the given targetironment

     */

    public void layout(LayoutTarget target, Graph g) {

        for(Iterator ns = g.nodes(); ns.hasNext(); ) {

            Node n = (Node)ns.next();

            if(target.isNodeVisible(n)) {

                Rectangle2D vp = target.getViewport(g);

                Rectangle2D bounds = target.getBounds(n);

                for(int i = 0; i < NUM_ITER; i++) {

                    // @author Claudia
          	    double x;
                    double y;

            	    if (n instanceof Noeud ){
            		x = ((Noeud)n).getCoordX();
            		y = ((Noeud)n).getCoordY();
            		}
            	     else {
                	x = vp.getX() + Math.abs(Math.random())*vp.getWidth();

                	y = vp.getY() + Math.abs(Math.random())*vp.getHeight();
                	}

                    LayoutUtilities.place(target, n, x, y);

                    bounds = target.getBounds(n);

                    boolean overlap = false;

                    Iterator j = target.intersectingNodes(bounds);

                     while(j.hasNext()) {

                        Node n2 = (Node)j.next();

                        if(n != n2) { overlap = false; }

                    }

                    if(!overlap) {

                        break;

                    }

                }

            }

        }

    }

}

