/*

 * $Id: BasicCompositeNode.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */



package diva.graph.model;

import diva.util.BasicPropertyContainer;

import java.util.*;

import java.io.*;



/**

 * A basic implementation of the Graph interface.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Yellow

 */

public class BasicCompositeNode extends BasicNode implements CompositeNode {

    /**

     * The storage for the Node contents of this graph.

     */

    private Vector _nodes;



    /**

     * Construct an empty graph.

     */

    public BasicCompositeNode() {

        this(null);

    }



    /**

     * Construct an empty graph with the specified user object.

     */

    public BasicCompositeNode(Object userObject) {

        super(userObject);

    	_nodes = new Vector();

    }



    public void add(BasicNode n) {

    	_nodes.addElement(n);

    	n.setParent(this);

    }



    public boolean contains(Node n) {

        return _nodes.contains(n);

    }



    public void clear() {

        while(getNodeCount() > 0) {

            BasicNode n = (BasicNode)getNode(0);

            delete(n);

        }

    }



    public void delete(BasicNode n) {

        n.getInEdgeSet().clear();

        n.getOutEdgeSet().clear();

    	_nodes.removeElement(n);

    }



    public int getNodeCount() {

        return _nodes.size();

    }



    public Node getNode(int i) {

        return (Node)_nodes.elementAt(i);

    }



    public int getIndex(Node n) {

        return _nodes.indexOf(n);

    }



    public Iterator nodes() {

        return _nodes.iterator();

    }



    public void remove(BasicNode n) {

        _nodes.removeElement(n);

    }



    public void removeAll() {

        _nodes.removeAllElements();

    }



    public String toString() {

        Object o = getSemanticObject();

        if(o != null) {

            return "BasicCompositeNode[" + o.toString() + "]";

        }

        else {

            return super.toString();

        }

    }

}

