/*

 * $Id: BasicEdge.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.model;

import diva.util.BasicPropertyContainer;

import java.util.*;

import java.io.Serializable;



/**

 * A basic implementation of the Edge interface.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Yellow

 */

public class BasicEdge extends BasicPropertyContainer implements Edge, Serializable {

    /**

     * The head of the edge.

     */

    private BasicNode _head = null;



    /**

     * Edge user data.

     */

    private Object _semanticObject = null;



    /**

     * The tail of the edge.

     */

    private BasicNode _tail = null;



    /**

     * Whether or not this edge is directed.

     */

    private boolean _directed = true;



    /**

     * The weight on this edge.

     */

    private double _weight = 1.0;



    /**

     * Create a new edge with no tail or head.

     */

    public BasicEdge() {

        this(null);

    }



    /**

     * Create a new edge with the specified user object

     * but no tail or head.

     */

    public BasicEdge(Object userObject) {

        this(userObject, null, null);

    }



    /**

     * Create a new edge with the specified tail/head.

     */

    public BasicEdge(BasicNode tail, BasicNode head) {

        this(null, tail, head);

    }



    /**

     * Create a new edge with the specified user object

     * and tail/head

     */

    public BasicEdge(Object userObject, BasicNode tail, BasicNode head) {

        setSemanticObject(userObject);

        attach(tail, head);

    }



    public void attach(BasicNode tail, BasicNode head) {

        setTail(tail);

        setHead(head);

    }



    public void detach() {

        setTail(null);

        setHead(null);

    }



    public Node getHead() { return _head; }

    

    public Node getTail() { return _tail; }

    

    public double getWeight() { return _weight; }

    

    public Object getSemanticObject() { return _semanticObject; }



    public boolean isDirected() { return _directed; }

    

    public void setDirected(boolean val) {

	_directed = val;

    }



    public void setHead(BasicNode n) {

        BasicNode prevHead = (BasicNode)getHead();

	if(prevHead != null) {

	    prevHead.getInEdgeSet().remove(this);

	}



	_head = n;

        if(_head != null) {

    	    _head.getInEdgeSet().add(this);

        }

    }



    public void setTail(BasicNode n) {

        BasicNode prevTail = (BasicNode)getTail();

    	if(prevTail != null) {

	    prevTail.getOutEdgeSet().remove(this);

	}



	_tail = n;

        if(_tail != null) {

    	    _tail.getOutEdgeSet().add(this);

        }

    }



    public void setWeight(double weight ) { _weight = weight; }



    public void setSemanticObject(Object o) { _semanticObject = o;  }



    public String toString() {

        Object o = getSemanticObject();

        if(o != null) {

            return "BasicEdge[" + o.toString() + "]";

        }

        else {

            return super.toString();

        }

    }

}

