/*

 * $Id: BasicGraph.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */



package diva.graph.model;

import diva.util.BasicPropertyContainer;

import java.util.*;

import java.awt.*;

import java.io.*;



/**

 * A basic implementation of the Graph interface.

 *

 * @author  Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating  Yellow

 */

public class BasicGraph extends BasicPropertyContainer implements IndexedGraph, Serializable {

    /**

     * User annotation.

     */

    private Object _semanticObject = null;



    /**

     * The storage for the Node contents of this graph.

     */

    private ArrayList _nodes;



    /**

     * Construct an empty graph.

     */

    public BasicGraph() {

        this(null);

    }



    /**

     * Construct an empty graph with the specified user object.

     */

    public BasicGraph(Object userObject) {

        setSemanticObject(userObject);

    	_nodes = new ArrayList();

    }



    /**

     * Construct an empty graph with some preallocated space

     * for nodes.

     *

     * @param n		Preallocated number of nodes.

     */

    public BasicGraph(int n) {

        this(null);

    	_nodes = new ArrayList(n);

    }





    public void add(BasicNode n) {

    	_nodes.add(n);

    	n.setParent(this);

    }



    public boolean contains(Node n) {

        return _nodes.contains(n);

    }



    public void clear() {

        while(getNodeCount() > 0) {

            BasicNode n = (BasicNode)getNode(0);

            delete(n);

        }

    }



    public void delete(BasicNode n) {

        n.getInEdgeSet().clear();

        n.getOutEdgeSet().clear();

	remove(n);

    }



    public int getNodeCount() {

        return _nodes.size();

    }



    public Node getNode(int i) {

        return (Node)_nodes.get(i);

    }



    public int getIndex(Node n) {

        return _nodes.indexOf(n);

    }



    public Object getSemanticObject() {

	return _semanticObject;

    }



    



    public Iterator nodes() {

        return _nodes.iterator();

    }



    public void remove(BasicNode n) {

	int index = _nodes.indexOf(n);

	_nodes.remove(index);

    }



    public void removeAll() {

        _nodes.clear();

    }



    public void setSemanticObject(Object o) {

	_semanticObject = o;

    }



    public String toString() {

        Object o = getSemanticObject();

        if(o != null) {

            return "BasicGraph[" + o.toString() + "]";

        }

        else {

            return super.toString();

        }

    }

}

