/*

 * $Id: BasicNode.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.model;

import java.io.*;

import java.util.*;



/**

 * A basic implementation of the Node interface.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Yellow

 */

public class BasicNode extends diva.util.BasicPropertyContainer implements Node, Serializable {

    /**

     * User annotation.

     */

    private Object _semanticObject = null;



    /**

     * The graph to which this node belongs.

     */

    private Graph _parent = null;



    /**

     * The edges <b>into</b> this node.

     */

    private BasicEdgeSet _in = new BasicEdgeSet();



    /**

     * The edges <b>out of</b> this node.

     */

    private BasicEdgeSet _out = new BasicEdgeSet();



    /**

     * Whether or not this node has been visited.

     */

    private boolean _visited = false;





    /**

     * Create a completely empty node.

     */

    public BasicNode() {

	this(null);

    }



    /**

     * Create a node containing some user data.

     */

    public BasicNode(Object userObject) {

	setSemanticObject(userObject);

    }



    public BasicEdgeSet getInEdgeSet() {

        return _in;

    }



    public BasicEdgeSet getOutEdgeSet() {

        return _out;

    }


    public void setInEdgeSet(BasicEdgeSet bes) {

        _in=bes;

    }



    public void setOutEdgeSet(BasicEdgeSet bes) {

        _out=bes;

    }



    public Graph getParent() {

        return _parent;

    }



    public Object getSemanticObject() {

	return _semanticObject;

    }

    

    public boolean isVisited() {

	return _visited;

    }



    public Iterator inEdges() {

        return getInEdgeSet().edges();

    }





    public Iterator inNodes() {

        return new diva.util.IteratorAdapter () {

	    Iterator edges = getInEdgeSet().edges();

	    public boolean hasNext() {

		return edges.hasNext();

	    }

	    public Object next() {

		return ((Edge)edges.next()).getTail();

	    }

	};

    }



    public Iterator outEdges() {

        return getOutEdgeSet().edges();

    }



    public Iterator outNodes() {

        return new diva.util.IteratorAdapter () {

	    Iterator edges = getOutEdgeSet().edges();

	    public boolean hasNext() {

		return edges.hasNext();

	    }

	    public Object next() {

		return ((Edge)edges.next()).getHead();

	    }

	};

    }



    public void setParent(Graph g) {

        _parent = g;

    }



    public void setSemanticObject(Object o) {

	_semanticObject = o;

    }



    public void setVisited(boolean val) {

	_visited = val;

    }



    public String toString() {

        Object o = getSemanticObject();

        if(o != null) {

            return "BasicNode[" + o.toString() + "]";

        }

        else {

            return super.toString();

        }

    }

}



