/*

 * $Id: Edge.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.model;

import diva.util.PropertyContainer;

import diva.util.SemanticObjectContainer;



/**

 * An edge between two nodes in a graph.  Edges connect a "tail" to a

 * "head" node; hyperedges can be simulated by having special nodes

 * which represent the hyperedge. An edge can be annotated with a user

 * object and arbitrary properties, and can be tagged as directed or

 * undirected.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Yellow

 */

public interface Edge extends PropertyContainer, SemanticObjectContainer {

    /**

     * Return the head node of this edge.

     */

    public Node getHead();

    

    /**

     * Return the tail node of this edge.

     */

    public Node getTail();



    /**

     * Return whether or not this edge is directed.

     */

    public boolean isDirected();



    /**

     * Set whether or not this edge is directed.

     */

    public void setDirected(boolean val);

}





