/*

 * $Id: ContainmentModel.java,v 1.1.1.1 2000/05/15 22:11:08 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */



package diva.graph.toolbox;

import diva.graph.model.*;

import diva.util.*;

import diva.util.gui.TutorialWindow;

import javax.swing.JTree;

import javax.swing.event.*;

import javax.swing.tree.*;

import java.util.*;



/**

 * A class which models the containment relationship in the graph,

 * allowing it to be viewed in a swing tree widget.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Red

 */

public class ContainmentModel implements TreeModel {

    /**

     * The corresponding graph model.

     */

    private GraphModel _graphModel;



    /**

     * The corresponding graph model.

     */

    private EventListenerList _listeners = new EventListenerList();



    /**

     * A listener for the graph model.

     */

    private LocalListener _local = new LocalListener();



    /**

     * Construct a new containment model using

     * the given graph model.

     */

    public ContainmentModel(GraphModel m) {

        _graphModel = m;

        _graphModel.addGraphListener(_local);

    }



    public void addTreeModelListener(TreeModelListener l) {

        _listeners.add(TreeModelListener.class, l);

    }



    public Object getChild(Object parent, int index) {

        IndexedGraph g = (IndexedGraph)parent;

        return g.getNode(index);

    }



    public int getChildCount(Object parent) {

        if(parent instanceof IndexedGraph) {

            IndexedGraph g = (IndexedGraph)parent;

            return g.getNodeCount();

        }

        else {

            throw new ClassCastException("getChildCount: given object not a Graph!");

        }

    }



    public int getIndexOfChild(Object parent, Object child) {

        IndexedGraph g = (IndexedGraph)parent;

        Node n = (Node)child;

        return g.getIndex(n);

    }



    public Object getRoot() {

        return _graphModel.getGraph();

    }



    public boolean isLeaf(Object node) {

        if(!(node instanceof Node) && !(node instanceof IndexedGraph)) {

            throw new ClassCastException("isLeaf: given object not a Node or IndexedGraph");

        }

        boolean val = !(node instanceof IndexedGraph);

//        System.out.println("Is a leaf: " + val);

        return val;

    }



    public void removeTreeModelListener(TreeModelListener l) {

        _listeners.remove(TreeModelListener.class, l);

    }



// Messaged when the user has altered the value for the item identified by path to newValue.

// If newValue signifies a truly new value the model should post a treeNodesChanged event.

    public void valueForPathChanged(TreePath path, Object newValue) {

        System.out.println("valueForPathChanged() not implemented.");

        //XXX ignore this for now.

    }



    private void dispatchInserted(TreeModelEvent e) {

        Object[] listeners = _listeners.getListenerList();

        for (int i = listeners.length-2; i>=0; i-=2) {

            if (listeners[i] == TreeModelListener.class) {

                ((TreeModelListener)listeners[i+1]).treeNodesInserted(e);

            }

        }

    }



    private void dispatchRemoved(TreeModelEvent e) {

        Object[] listeners = _listeners.getListenerList();

        for (int i = listeners.length-2; i>=0; i-=2) {

            if (listeners[i] == TreeModelListener.class) {

                ((TreeModelListener)listeners[i+1]).treeNodesRemoved(e);

            }

        }

    }



    private void dispatchChanged(TreeModelEvent e) {

        Object[] listeners = _listeners.getListenerList();

        for (int i = listeners.length-2; i>=0; i-=2) {

            if (listeners[i] == TreeModelListener.class) {

                ((TreeModelListener)listeners[i+1]).treeNodesChanged(e);

            }

        }

    }



    private void dispatchStructureChanged(TreeModelEvent e) {

        Object[] listeners = _listeners.getListenerList();

        for (int i = listeners.length-2; i>=0; i-=2) {

            if (listeners[i] == TreeModelListener.class) {

                ((TreeModelListener)listeners[i+1]).treeStructureChanged(e);

            }

        }

    }



    private class LocalListener extends GraphAdapter {

        private TreePath makePath(Graph g) {

            if(!(g instanceof Node) ||

                (((Node)g).getParent() == null)) {

                return new TreePath(g);

            }

            else {

                Node n = (Node)g;

                TreePath p = makePath(n.getParent());

                return p.pathByAddingChild(n);

            }

        }



        public void nodeAdded(GraphEvent e) {

            TreePath p = makePath(((Node)e.getSource()).getParent());

            TreeModelEvent te = new TreeModelEvent(this, p);

//            dispatchInserted(te);

            dispatchStructureChanged(te);

        }



        public void nodeRemoved(GraphEvent e) {

            TreePath p = makePath(((Node)e.getSource()).getParent());

            TreeModelEvent te = new TreeModelEvent(this, p);

//            dispatchRemoved(te);

            dispatchStructureChanged(te);

        }



        public void structureChanged(GraphEvent e) {

            TreePath p = makePath(((Node)e.getSource()).getParent());

            TreeModelEvent te = new TreeModelEvent(this, p);

            dispatchStructureChanged(te);

        }

    }



    /**

     * Demonstrate this thing in action.

     */

    public static void main(String argv[]) {

        // Construct the widgets

        TutorialWindow f = new TutorialWindow("Tree");

        GraphModel m = new GraphModel();

        ContainmentModel cm = new ContainmentModel(m);

        JTree t = new JTree(cm);

        f.getContentPane().add("Center", t);

        f.setSize(800, 600);

        f.setVisible(true);



        // Construct the graph

        BasicNode a = new BasicNode("a");

        BasicNode b = new BasicNode("b");

        m.addNode(a);

        m.addNode(b);



        BasicCompositeNode c = new BasicCompositeNode("c");

        BasicNode d = new BasicNode("d");

        BasicNode e = new BasicNode("e");

        m.addNode(c);

        m.addNode(d, c);

        m.addNode(e, c);

    }

}

