/*

 * $Id: BasicPropertyContainer.java,v 1.1.1.1 2000/05/15 22:11:08 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.util;

import java.util.Hashtable;

import java.util.Iterator;



/**

 * An object that can be annotated with arbitrary

 * objects whose keys are strings.

 *

 * @author Michael Shilman (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 */

public class BasicPropertyContainer implements PropertyContainer {

    /**

     * The default values, if applicable.

     */

    PropertyContainer _defaults;



    /**

     * The mapping from keys to values.

     */

    Hashtable _mapping;



    /**

     * A property container with no defaults.

     */

    public BasicPropertyContainer() {

        this(null);

    }



    /**

     * A property container with no defaults.

     */

    public BasicPropertyContainer(PropertyContainer defaults) {

        _defaults = defaults;

    }



    /**

     * Return the property corresponding to

     * the given key, or null if no such property

     * exists.

     */

    public Object getProperty(String key) {

        Object o = null;

        if(_mapping != null) {

            o = _mapping.get(key);

        }

        if(o == null && _defaults != null) {

            return _defaults.getProperty(key);

        }

        return o;

    }



    /**

     * Set the property corresponding to

     * the given key.

     */

    public void setProperty(String key, Object value) {

        if(_mapping == null) {

           _mapping = new Hashtable();

        }

        _mapping.put(key, value);

    }



    //XXX remove property

    //added by Heloise

    public void removeAllProperties(){

        _mapping.clear();

    }



    //added by Heloise

    public Iterator properties(){

        return _mapping.values().iterator();

    }



}

