/*

 * $Id: Node.java,v 1.1.1.1 2000/05/15 22:11:07 dauplait Exp $

 *

 * Copyright (c) 1998 The Regents of the University of California.

 * All rights reserved.  See the file COPYRIGHT for details.

 */

package diva.graph.model;

import java.util.Iterator;

import diva.util.PropertyContainer;

import diva.util.SemanticObjectContainer;



/**

 * A node in a graph which can be connected by edges.

 * Nodes' edges are divided into "in" and "out" edges, which is

 * not fundamental but was chosen because it is easier to

 * implement an undirected graph using this scheme than

 * to implement a directed graph using a more general

 * scheme.  Furthermore, algorithms on edges can avoid having

 * to mark an edge as visited by exploiting this structure:

 * an algorithm can iterate over the nodes in the graph

 * and act on only the OUT edges for each node, ensuring

 * that each edge is visited once.  <p>

 *

 * Nodes can be annotated with a user object

 * and arbitrary properties, and have a visited

 * flag.

 *

 * @author Michael Shilman  (michaels@eecs.berkeley.edu)

 * @version $Revision: 1.1.1.1 $

 * @rating Yellow

 */

public interface Node extends PropertyContainer, SemanticObjectContainer {

    /**

     * Return an iterator over the <i>in</i> edges of this

     * node. This iterator does not support removal operations.

     * If there are no in-edges, an iterator with no elements is

     * returned.

     */

    public Iterator inEdges();



    /**

     * Return an iterator over the <i>out</i> edges of this

     * node.  This iterator does not support removal operations.

     * If there are no in-edges, an iterator with no elements is

     * returned.

     */

    public Iterator outEdges();



    /**

     * Return the parent graph of this node.

     */

    public Graph getParent();



    /**

     * Return the visited flag for this node.  This might be

     * used by traversal algorithms for a cyclic graph.

     */

    public boolean isVisited();



    /**

     * Set the visited flag for this node.  Algorithms are

     * responsible for setting the visited flag to "false"

     * after before they begin a traversal (in other words,

     * they cannot expect that a previous traversal has

     * left the nodes unmarked).

     */

    public void setVisited(boolean val);

}

